#    Copyright (C) 2005 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

"""
Numerical fitting of functions to data.
"""

import sys

import numpy as N
try:
    import numpy.linalg as NLA
except:
    import scipy.linalg as NLA

def fitLM(func, params, xvals, yvals, errors,
          stopdeltalambda = 1e-5,
          deltaderiv = 1e-5, maxiters = 20, Lambda = 1e-4):

    """
    Use Marquardt method as described in Bevington & Robinson to fit data

    func is a python function to evaluate. It takes two parameters, the
    parameters to fit as a numpy, and the values of x to evaluate the
    function at

    params is a numpy of parameters to fit for. These are passed to
    the function.

    xvals are x data points (numpy), yvals are the y data points (numarray)
    errors are a numpy of errors on the y data points.
    Set all to 1 if not important.

    stopdeltalambda: minimum change in chi2 to carry on fitting
    deltaderiv: change to make in parameters to calculate derivative
    maxiters: maximum number of better fitting solutions before stopping
    Lambda: starting lambda value (as described in Bevington)
    """

    # optimisation to avoid computing this all the time
    inve2 = 1. / errors**2

    # work out fit using current parameters
    oldfunc = func(params, xvals)
    chi2 = ( (oldfunc - yvals)**2 * inve2 ).sum()

    # initialise temporary space
    beta = N.zeros( len(params), dtype='float64' )
    alpha = N.zeros( (len(params), len(params)), dtype='float64' )
    derivs = N.zeros( (len(params), len(xvals)), dtype='float64' )

    done = False
    iters = 0
    while iters < maxiters and not done:
        # iterate over each of the parameters and calculate the derivatives
        # of chi2 to populate the beta vector

        # also calculate the derivative of the function at each of the points
        # wrt the parameters

        for i in range( len(params) ):
            params[i] += deltaderiv
            new_func = func(params, xvals)
            chi2_new = ((new_func - yvals)**2 * inve2).sum()
            params[i] -= deltaderiv

            beta[i] = chi2_new - chi2
            derivs[i] = new_func - oldfunc

        # beta is now dchi2 / dparam
        beta *= (-0.5 / deltaderiv)
        derivs *= (1. / deltaderiv)

        # calculate alpha matrix
        # FIXME: stupid - must be a better way to avoid this iteration
        for j in range( len(params) ):
            for k in range(j+1):
                v = (derivs[j]*derivs[k] * inve2).sum()
                alpha[j][k] = v
                alpha[k][j] = v

        # twiddle alpha using lambda
        alpha *= 1. + N.identity(len(params), dtype='float64')*Lambda

        # now work out deltas on parameters to get better fit
        deltas = NLA.solve(alpha, beta)

        # new solution
        new_params = params+deltas
        new_func = func(new_params, xvals)
        new_chi2 = ( (new_func - yvals)**2 * inve2 ).sum()

        if N.isnan(new_chi2):
            sys.stderr.write('Chi2 is NaN. Aborting fit.\n')
            break

        if new_chi2 > chi2:
            # if solution is worse, increase lambda
            Lambda *= 10.
        else:
            # better fit, so we accept this solution

            # if the change is small
            done = chi2 - new_chi2 < stopdeltalambda

            chi2 = new_chi2
            params = new_params
            oldfunc = new_func
            Lambda *= 0.1

            # format new parameters
            iters += 1
            p = [iters, chi2] + params.tolist()
            str = ("%5i " + "%8g " * (len(params)+1)) % tuple(p)
            print(str)

    if not done:
        sys.stderr.write("Warning: maximum number of iterations reached\n")

    # print out fit statistics at end
    dof = len(yvals) - len(params)
    redchi2 = chi2 / dof
    print("chi^2 = %g, dof = %i, reduced-chi^2 = %g" % (chi2, dof, redchi2))

    return (params, chi2, dof)



#    Copyright (C) 2010 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

import re
import math
import numpy as N

from . import dates

_formaterror = 'FormatError'

# a format statement in a string
_format_re = re.compile(r'%([-#0-9 +.hlL]*?)([diouxXeEfFgGcrs%])')

def localeFormat(totfmt, args, locale=None):
    """Format using fmt statement fmt, qt QLocale object locale and
    arguments to formatting args.

    * arguments are not supported in this formatting, nor is using
    a dict to supply values for statement
    """

    # substitute all format statements with string format statements
    newfmt = _format_re.sub("%s", totfmt)

    # do formatting separately for all statements
    strings = []
    i = 0
    for f in _format_re.finditer(totfmt):
        code = f.group(2)
        if code == '%':
            s = '%'
        else:
            try:
                s = f.group() % args[i]
                i += 1
            except IndexError:
                raise TypeError("Not enough arguments for format string")
            s = s.replace('-', '\u2212')
            if locale is not None and code in 'eEfFgG':
                s = s.replace('.', locale.decimalPoint())

        strings.append(s)

    if i != len(args):
        raise TypeError("Not all arguments converted during string formatting")

    return newfmt % tuple(strings)

def sciToHuman(val, cleanup=False):
    """Convert output from C formatting to human scientific notation.
    if cleanup, remove zeros after decimal points
    """

    # split around the exponent
    leader, exponent = val.split('e')

    # strip off trailing decimal point and zeros if no format args
    if cleanup and leader.find('.') >= 0:
        leader = leader.rstrip('0').rstrip('.')

    # trim off leading 1
    if leader == '1' and cleanup:
        leader = ''
    else:
        # add multiply sign
        leader += '\u00d7'

    return '%s10^{%i}' % (leader, int(exponent))

def formatSciNotation(num, formatargs, locale=None):
    """Format number into form X \times 10^{Y}.
    This function trims trailing zeros and decimal point unless a formatting
    argument is supplied

    This is similar to the %e format string
    formatargs is the standard argument in a format string to control the
    number of decimal places, etc.

    locale is a QLocale object
    """

    # handle nan, inf, -inf
    if not N.isfinite(num):
        return str(num)

    # create an initial formatting string
    if formatargs:
        formatstr = '%' + formatargs + 'e'
    else:
        formatstr = '%.10e'

    # do formatting, catching errors
    try:
        text = formatstr % num
    except:
        return _formaterror

    text = sciToHuman(text, cleanup=formatargs=='')

    # do substitution of decimals
    if locale is not None:
        text = text.replace('.', locale.decimalPoint())

    return text

def formatGeneral(num, fmtarg, locale=None):
    """General formatting which switches from normal to scientic
    notation."""

    if fmtarg:
        # if an argument is given, we convert output
        try:
            retn = ('%'+fmtarg+'g') % num
        except ValueError:
            retn = _formaterror
        if retn.find('e') >= 0:
            # in scientific notation, so convert
            retn = sciToHuman(retn, cleanup=False)
    else:
        a = abs(num)
        # manually choose when to switch from normal to scientific
        # as the default %g isn't very good
        if a >= 1e4 or (a < 1e-2 and a > 1e-110):
            retn = formatSciNotation(num, fmtarg, locale=locale)
        else:
            retn = '%.10g' % num

    if locale is not None:
        # replace decimal point with correct decimal point
        retn = retn.replace('.', locale.decimalPoint())
    return retn

engsuffixes = (
    'y', 'z', 'a', 'f', 'p', 'n',
    '\u03bc', 'm', '', 'k', 'M', 'G',
    'T', 'P', 'E', 'Z', 'Y'
)

def formatEngineering(num, fmtarg, locale=None):
    """Engineering suffix format notation using SI suffixes."""

    if num != 0.:
        logindex = math.log10( abs(num) ) / 3.

        # for numbers < 1 round down suffix
        if logindex < 0. and (int(logindex)-logindex) > 1e-6:
            logindex -= 1

        # make sure we don't go out of bounds
        logindex = min( max(logindex, -8),
                        len(engsuffixes) - 9 )

        suffix = engsuffixes[ int(logindex) + 8 ]
        val = num / 10**( int(logindex) *3)
    else:
        suffix = ''
        val = num

    text = ('%' + fmtarg + 'g%s') % (val, suffix)
    if locale is not None:
        text = text.replace('.', locale.decimalPoint())
    return text

# catch general veusz formatting expression
_formatRE = re.compile(r'%([-0-9.+# ]*)(VDVS|VD.|V.|[A-Za-z%])')

def formatNumber(num, formatstr, locale=None):
    """ Format a number in different ways.

    formatstr is a standard C format string, with some additions:
     %Ve    scientific notation X \times 10^{Y}
     %Vg    switches from normal notation to scientific outside 10^-2 to 10^4
     %VE    engineering suffix option

     %VDx   date formatting, where x is one of the arguments in
            http://docs.python.org/lib/module-time.html in the function
            strftime
    """

    outitems = []
    while formatstr:
        # repeatedly try to do string format
        match = _formatRE.search(formatstr)
        if not match:
            outitems.append(formatstr)
            break

        # argument and type of formatting
        farg, ftype = match.groups()

        # special veusz formatting
        if ftype[:1] == 'V':
            # special veusz formatting
            if ftype == 'Ve':
                out = formatSciNotation(num, farg, locale=locale)
            elif ftype == 'Vg':
                out = formatGeneral(num, farg, locale=locale)
            elif ftype == 'VE':
                out = formatEngineering(num, farg, locale=locale)
            elif ftype[:2] == 'VD':
                d = dates.floatToDateTime(num)
                # date formatting (seconds since start of epoch)
                if ftype[:4] == 'VDVS':
                    # special seconds operator
                    out = ('%'+ftype[4:]+'g') % (d.second+d.microsecond*1e-6)
                else:
                    # use date formatting
                    try:
                        out = d.strftime(str('%'+ftype[2:]))
                    except ValueError:
                        out = _formaterror
            else:
                out = _formaterror

            # replace hyphen with true minus sign
            out = out.replace('-', '\u2212')
        elif ftype == '%':
            out = '%'
        else:
            # standard C formatting
            try:
                out = localeFormat('%' + farg + ftype, (num,), locale=locale)
            except:
                out = _formaterror

        outitems.append(formatstr[:match.start()])
        outitems.append(out)
        formatstr = formatstr[match.end():]

    return ''.join(outitems)



#    Copyright (C) 2013 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

import re

from .. import qtall as qt
from .. import document
from . import readcsv
from . import base

def _(text, disambiguation=None, context="Import_CSV"):
    return qt.QCoreApplication.translate(context, text, disambiguation)

class ImportParamsCSV(base.ImportParamsBase):
    """CSV import parameters.

    additional parameters:
     readrows: read data in rows
     delimiter: CSV delimiter
     skipwhitespace: whether to ignore white space following CSV delimiter
     textdelimiter: delimiter for text
     headerignore: number of lines to ignore after headers
     rowsignore: number of lines to ignore at top of file
     blanksaredata: treat blank entries as nans
     numericlocale: name of local for numbers
     dateformat: date format string
     headermode: 'multi', '1st' or 'none'
    """

    defaults = {
        'readrows': False,
        'delimiter': ',',
        'skipwhitespace' : False,
        'textdelimiter': '"',
        'headerignore': 0,
        'rowsignore': 0,
        'blanksaredata': False,
        'numericlocale': 'en_US',
        'dateformat': 'YYYY-MM-DD|T|hh:mm:ss',
        'headermode': 'multi',
    }
    defaults.update(base.ImportParamsBase.defaults)

    def __init__(self, **argsv):
        base.ImportParamsBase.__init__(self, **argsv)
        if self.headermode not in ('multi', '1st', 'none'):
            raise ValueError("Invalid headermode")

class OperationDataImportCSV(base.OperationDataImportBase):
    """Import data from a CSV file."""

    descr = _('import CSV data')

    def doImport(self):
        """Do the data import."""

        try:
            csvr = readcsv.ReadCSV(self.params)
        except re.error:
            # invalid date RE
            raise base.ImportingError(_('Invalid date regular expression'))

        csvr.readData()

        LF = None
        if self.params.linked:
            LF = LinkedFileCSV(self.params)

        # set the data in the output structure
        csvr.setData(self.outdatasets, linkedfile=LF)

class LinkedFileCSV(base.LinkedFileBase):
    """A CSV file linked to datasets."""

    def createOperation(self):
        """Return operation to recreate self."""
        return OperationDataImportCSV

    def saveToFile(self, fileobj, relpath=None):
        """Save the link to the document file."""
        self._saveHelper(
            fileobj,
            'ImportFileCSV',
            ('filename',),
            renameparams={'prefix': 'dsprefix', 'suffix': 'dssuffix'},
            relpath=relpath)

def ImportFileCSV(comm, filename,
                  readrows=False,
                  delimiter=',', skipwhitespace=False, textdelimiter='"',
                  encoding='utf_8',
                  headerignore=0, rowsignore=0,
                  blanksaredata=False,
                  numericlocale='en_US',
                  dateformat='YYYY-MM-DD|T|hh:mm:ss',
                  headermode='multi',
                  dsprefix='', dssuffix='', prefix=None,
                  renames=None,
                  linked=False):
    """Read data from a comma separated file (CSV).

    Data are read from filename

    readrows: if true, data are read across rather than down
    delimiter: character for delimiting data (usually ',')
    skipwhitespace: if true, white space following delimiter is ignored
    textdelimiter: character surrounding text (usually '"')
    encoding: encoding used in file
    headerignore: number of lines to ignore after header text
    rowsignore: number of rows to ignore at top of file
    blanksaredata: treats blank lines in csv files as blank data values
    numericlocale: format to use for reading numbers
    dateformat: format for interpreting dates
    headermode: 'multi': multiple headers allowed in file
                '1st': first text found are headers
                'none': no headers, guess data and use default names

    Dataset names are prepended and appended, by dsprefix and dssuffix,
    respectively
     (prefix is backware compatibility only, it adds an underscore
      relative to dsprefix)

    renames is a map of old names to new names to rename on import

    If linked is True the data are linked with the file.

    Returns: list of imported datasets
    """

    # backward compatibility
    if prefix:
        dsprefix = prefix + '_'

    # lookup filename
    realfilename = comm.findFileOnImportPath(filename)

    params = ImportParamsCSV(
        filename=realfilename, readrows=readrows,
        delimiter=delimiter, skipwhitespace=skipwhitespace,
        textdelimiter=textdelimiter,
        encoding=encoding,
        headerignore=headerignore, rowsignore=rowsignore,
        blanksaredata=blanksaredata,
        numericlocale=numericlocale, dateformat=dateformat,
        headermode=headermode,
        prefix=dsprefix, suffix=dssuffix,
        renames=renames,
        linked=linked,
    )
    op = OperationDataImportCSV(params)
    comm.document.applyOperation(op)

    if comm.verbose:
        print("Imported datasets %s" % ' '.join(op.outnames))
    return op.outnames

document.registerImportCommand('ImportFileCSV', ImportFileCSV)



#    Copyright (C) 2009 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

"""A combobox which remembers previous setting
"""

from .. import qtall as qt
from .. import setting

class HistoryValueCombo(qt.QComboBox):
    """This combobox records what value was previously saved
    """

    def __init__(self, *args):
        qt.QComboBox.__init__(self, *args)
        self.defaultlist = []
        self.defaultval = None
        self.hasshown = False

    def getSettingName(self):
        """Get name for saving in settings."""

        # get dialog for widget
        dialog = self.parent()
        while not isinstance(dialog, qt.QDialog):
            dialog = dialog.parent()

        # combine dialog and object names to make setting
        return '%s_%s_HistoryValueCombo'  % (
            dialog.objectName(), self.objectName() )

    def saveHistory(self):
        """Save contents of history combo to settings."""

        # only save history if it has been loaded
        if not self.hasshown:
            return

        # collect current items
        history = [ self.itemText(i) for i in range(self.count()) ]
        history.insert(0, self.currentText())

        # remove dups
        histout = []
        histset = set()
        for item in history:
            if item not in histset:
                histout.append(item)
                histset.add(item)

        # save the history
        setting.settingdb[self.getSettingName()] = histout

    def showEvent(self, event):
        """Show HistoryCombo and load history."""
        qt.QComboBox.showEvent(self, event)
        if self.hasshown:
            return

        self.clear()
        self.addItems(self.defaultlist)
        text = setting.settingdb.get(self.getSettingName(), self.defaultval)
        if text is not None:
            indx = self.findText(text)
            if indx < 0:
                if self.isEditable():
                    self.insertItem(0, text)
                indx = 0
            self.setCurrentIndex(indx)
        self.hasshown = True

    def hideEvent(self, event):
        """Save history as widget is hidden."""
        qt.QComboBox.hideEvent(self, event)

        if self.hasshown:
            text = self.currentText()
            setting.settingdb[self.getSettingName()] = text




#    Copyright (C) 2009 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

"""A paint engine to produce EMF exports.

Requires: PyQt-x11-gpl-4.6-snapshot-20090906.tar.gz
          sip-4.9-snapshot-20090906.tar.gz
          pyemf
"""

import struct

import pyemf3
import pyemf3.emr

from .. import qtall as qt

inch_mm = 25.4
scale = 100

def isStockObject(obj):
    """Is this a stock windows object."""
    return (obj & 0x80000000) != 0

class EMFPaintEngine(qt.QPaintEngine):
    """Custom EMF paint engine."""

    def __init__(self, width_in, height_in, dpi=75):
        qt.QPaintEngine.__init__(
            self,
            qt.QPaintEngine.Antialiasing |
            qt.QPaintEngine.PainterPaths |
            qt.QPaintEngine.PrimitiveTransform |
            qt.QPaintEngine.PaintOutsidePaintEvent |
            qt.QPaintEngine.PatternBrush
        )
        self.width = width_in
        self.height = height_in
        self.dpi = dpi

    def begin(self, paintdevice):
        self.emf = pyemf3.EMF(self.width, self.height, int(self.dpi*scale))
        self.pen = self.emf.GetStockObject(pyemf3.BLACK_PEN)
        self.pencolor = (0, 0, 0)
        self.brush = self.emf.GetStockObject(pyemf3.NULL_BRUSH)

        self.paintdevice = paintdevice
        return True

    def drawLines(self, lines):
        """Draw lines to emf output."""

        for line in lines:
            self.emf.Polyline(
                 [ (int(line.x1()*scale), int(line.y1()*scale)),
                   (int(line.x2()*scale), int(line.y2()*scale)) ] )

    def drawPolygon(self, points, mode):
        """Draw polygon on output."""
        # print "Polygon"
        pts = [(int(p.x()*scale), int(p.y()*scale)) for p in points]

        if mode == qt.QPaintEngine.PolylineMode:
            self.emf.Polyline(pts)
        else:
            self.emf.SetPolyFillMode({
                qt.QPaintEngine.WindingMode: pyemf3.WINDING,
                qt.QPaintEngine.OddEvenMode: pyemf3.ALTERNATE,
                qt.QPaintEngine.ConvexMode: pyemf3.WINDING
            })
            self.emf.Polygon(pts)

    def drawEllipse(self, rect):
        """Draw an ellipse."""
        # print "ellipse"
        args = (
            int(rect.left()*scale),  int(rect.top()*scale),
            int(rect.right()*scale), int(rect.bottom()*scale),
            int(rect.left()*scale),  int(rect.top()*scale),
            int(rect.left()*scale),  int(rect.top()*scale),
        )
        self.emf.Pie(*args)
        self.emf.Arc(*args)

    def drawPoints(self, points):
        """Draw points."""
        # print "points"

        for pt in points:
            x, y = (pt.x()-0.5)*scale, (pt.y()-0.5)*scale
            self.emf.Pie(
                int(x), int(y),
                int((pt.x()+0.5)*scale), int((pt.y()+0.5)*scale),
                int(x), int(y), int(x), int(y) )

    def drawPixmap(self, r, pixmap, sr):
        """Draw pixmap to display."""

        # convert pixmap to BMP format
        bytearr = qt.QByteArray()
        buf = qt.QBuffer(bytearr)
        buf.open(qt.QIODevice.WriteOnly)
        pixmap.save(buf, "BMP")

        bmp = bytes(buf.data())
        self.emf.BitmapOut(
            int(r.left()*scale), int(r.top()*scale),
            int(r.width()*scale), int(r.bottom()*scale),
            int(sr.left()), int(sr.top()),
            int(sr.width()), int(sr.height()),
            bmp,
        )

    def _createPath(self, path):
        """Convert qt path to emf path"""
        self.emf.BeginPath()
        count = path.elementCount()
        i = 0
        #print "Start path"
        while i < count:
            e = path.elementAt(i)
            if e.type == qt.QPainterPath.MoveToElement:
                self.emf.MoveTo( int(e.x*scale), int(e.y*scale) )
                #print "M", e.x*scale, e.y*scale
            elif e.type == qt.QPainterPath.LineToElement:
                self.emf.LineTo( int(e.x*scale), int(e.y*scale) )
                #print "L", e.x*scale, e.y*scale
            elif e.type == qt.QPainterPath.CurveToElement:
                e1 = path.elementAt(i+1)
                e2 = path.elementAt(i+2)
                params = (
                    ( int(e.x*scale), int(e.y*scale) ),
                    ( int(e1.x*scale), int(e1.y*scale) ),
                    ( int(e2.x*scale), int(e2.y*scale) ),
                )
                self.emf.PolyBezierTo(params)
                #print "C", params

                i += 2
            else:
                assert False

            i += 1

        ef = path.elementAt(0)
        el = path.elementAt(count-1)
        if ef.x == el.x and ef.y == el.y:
            self.emf.CloseFigure()
            #print "closing"
        self.emf.EndPath()

    def drawPath(self, path):
        """Draw a path on the output."""
        # print "path"

        self._createPath(path)
        self.emf.StrokeAndFillPath()

    def drawTextItem(self, pt, textitem):
        """Convert text to a path and draw it.
        """
        # print "text", pt, textitem.text()
        path = qt.QPainterPath()
        path.addText(pt, textitem.font(), textitem.text())

        fill = self.emf.CreateSolidBrush(self.pencolor)
        self.emf.SelectObject(fill)
        self._createPath(path)
        self.emf.FillPath()
        self.emf.SelectObject(self.brush)
        self.emf.DeleteObject(fill)

    def end(self):
        return True

    def saveFile(self, filename):
        self.emf.save(filename)

    def _updatePen(self, pen):
        """Update the pen to the currently selected one."""

        # line style
        style = {
            qt.Qt.NoPen: pyemf3.PS_NULL,
            qt.Qt.SolidLine: pyemf3.PS_SOLID,
            qt.Qt.DashLine: pyemf3.PS_DASH,
            qt.Qt.DotLine: pyemf3.PS_DOT,
            qt.Qt.DashDotLine: pyemf3.PS_DASHDOT,
            qt.Qt.DashDotDotLine: pyemf3.PS_DASHDOTDOT,
            qt.Qt.CustomDashLine: pyemf3.PS_USERSTYLE,
        }[pen.style()]

        if style != pyemf3.PS_NULL:
            # set cap style
            style |= {
                qt.Qt.FlatCap: pyemf3.PS_ENDCAP_FLAT,
                qt.Qt.SquareCap: pyemf3.PS_ENDCAP_SQUARE,
                qt.Qt.RoundCap: pyemf3.PS_ENDCAP_ROUND,
            }[pen.capStyle()]

            # set join style
            style |= {
                qt.Qt.MiterJoin: pyemf3.PS_JOIN_MITER,
                qt.Qt.BevelJoin: pyemf3.PS_JOIN_BEVEL,
                qt.Qt.RoundJoin: pyemf3.PS_JOIN_ROUND,
                qt.Qt.SvgMiterJoin: pyemf3.PS_JOIN_MITER,
            }[pen.joinStyle()]

            # use proper widths of lines
            style |= pyemf3.PS_GEOMETRIC

        width = int(pen.widthF()*scale)
        qc = pen.color()
        color = (qc.red(), qc.green(), qc.blue())
        self.pencolor = color

        if pen.style() == qt.Qt.CustomDashLine:
            # custom dash pattern
            dash = [int(pen.widthF()*scale*f) for f in pen.dashPattern()]
        else:
            dash = None

        newpen = self.emf.CreatePen(style, width, color, styleentries=dash)
        self.emf.SelectObject(newpen)

        # delete old pen if it is not a stock object
        if not isStockObject(self.pen):
            self.emf.DeleteObject(self.pen)
        self.pen = newpen

    def _updateBrush(self, brush):
        """Update to selected brush."""

        style = brush.style()
        qc = brush.color()
        color = (qc.red(), qc.green(), qc.blue())
        # print "brush", color
        if style == qt.Qt.SolidPattern:
            newbrush = self.emf.CreateSolidBrush(color)
        elif style == qt.Qt.NoBrush:
            newbrush = self.emf.GetStockObject(pyemf3.NULL_BRUSH)
        else:
            try:
                hatch = {
                    qt.Qt.HorPattern: pyemf3.HS_HORIZONTAL,
                    qt.Qt.VerPattern: pyemf3.HS_VERTICAL,
                    qt.Qt.CrossPattern: pyemf3.HS_CROSS,
                    qt.Qt.BDiagPattern: pyemf3.HS_BDIAGONAL,
                    qt.Qt.FDiagPattern: pyemf3.HS_FDIAGONAL,
                    qt.Qt.DiagCrossPattern: pyemf3.HS_DIAGCROSS
                }[brush.style()]
            except KeyError:
                newbrush = self.emf.CreateSolidBrush(color)
            else:
                newbrush = self.emf.CreateHatchBrush(hatch, color)
        self.emf.SelectObject(newbrush)

        if not isStockObject(self.brush):
            self.emf.DeleteObject(self.brush)
        self.brush = newbrush

    def _updateClipPath(self, path, operation):
        """Update clipping path."""
        # print "clip"
        if operation != qt.Qt.NoClip:
            self._createPath(path)
            clipmode = {
                 qt.Qt.ReplaceClip: pyemf3.RGN_COPY,
                 qt.Qt.IntersectClip: pyemf3.RGN_AND,
            }[operation]
        else:
            # is this the only wave to get rid of clipping?
            self.emf.BeginPath()
            self.emf.MoveTo(0,0)
            w = int(self.width*self.dpi*scale)
            h = int(self.height*self.dpi*scale)
            self.emf.LineTo(w, 0)
            self.emf.LineTo(w, h)
            self.emf.LineTo(0, h)
            self.emf.CloseFigure()
            self.emf.EndPath()
            clipmode = pyemf3.RGN_COPY

        self.emf.SelectClipPath(mode=clipmode)

    def _updateTransform(self, m):
        """Update transformation."""
        self.emf.SetWorldTransform(
            m.m11(), m.m12(),
            m.m21(), m.m22(),
            m.dx()*scale, m.dy()*scale)

    def updateState(self, state):
        """Examine what has changed in state and call apropriate function."""
        ss = state.state()
        if ss & qt.QPaintEngine.DirtyPen:
            self._updatePen(state.pen())
        if ss & qt.QPaintEngine.DirtyBrush:
            self._updateBrush(state.brush())
        if ss & qt.QPaintEngine.DirtyTransform:
            self._updateTransform(state.transform())
        if ss & qt.QPaintEngine.DirtyClipPath:
            self._updateClipPath(state.clipPath(), state.clipOperation())
        if ss & qt.QPaintEngine.DirtyClipRegion:
            path = qt.QPainterPath()
            path.addRegion(state.clipRegion())
            self._updateClipPath(path, state.clipOperation())

    def type(self):
        return qt.QPaintEngine.PostScript

class EMFPaintDevice(qt.QPaintDevice):
    """Paint device for EMF paint engine."""

    def __init__(self, width_in, height_in, dpi=75):
        qt.QPaintDevice.__init__(self)
        self.engine = EMFPaintEngine(width_in, height_in, dpi=dpi)

    def paintEngine(self):
        return self.engine

    def metric(self, m):
        """Return the metrics of the painter."""
        if m == qt.QPaintDevice.PdmWidth:
            return int(self.engine.width * self.engine.dpi)
        elif m == qt.QPaintDevice.PdmHeight:
            return int(self.engine.height * self.engine.dpi)
        elif m == qt.QPaintDevice.PdmWidthMM:
            return int(self.engine.width * inch_mm)
        elif m == qt.QPaintDevice.PdmHeightMM:
            return int(self.engine.height * inch_mm)
        elif m == qt.QPaintDevice.PdmNumColors:
            return 2147483647
        elif m == qt.QPaintDevice.PdmDepth:
            return 24
        elif m == qt.QPaintDevice.PdmDpiX:
            return int(self.engine.dpi)
        elif m == qt.QPaintDevice.PdmDpiY:
            return int(self.engine.dpi)
        elif m == qt.QPaintDevice.PdmPhysicalDpiX:
            return int(self.engine.dpi)
        elif m == qt.QPaintDevice.PdmPhysicalDpiY:
            return int(self.engine.dpi)
        elif m == qt.QPaintDevice.PdmDevicePixelRatio:
            return 1

        # Qt >= 5.6
        elif m == getattr(qt.QPaintDevice, 'PdmDevicePixelRatioScaled', -1):
            return 1

        else:
            # fall back
            return qt.QPaintDevice.metric(self, m)



#    Copyright (C) 2016 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

"""Widget for plotting data covariance."""

import numpy as N

from . import plotters
from .. import qtall as qt
from .. import document
from .. import setting
from .. import utils

def _(text, disambiguation=None, context='Covariance'):
    """Translate text."""
    return qt.QCoreApplication.translate(context, text, disambiguation)

class CovarianceLine(setting.Line):
    def __init__(self, name, **args):
        setting.Line.__init__(self, name, **args)

        self.add( setting.Int(
            'steps',
            25,
            minval=4,
            descr=_('Number of line steps to draw'),
            usertext=_('Steps') ))

        self.get('color').newDefault('auto')

class Covariance(plotters.GenericPlotter):
    """Plot covariance matrix for points as shapes."""

    typename = 'covariance'
    allowusercreation=True
    description=_('Plot covariance ellipses')

    def __init__(self, parent, **args):
        """Initialise plotter."""

        plotters.GenericPlotter.__init__(self, parent, **args)

        self._elpts = []
        self._changeset = -1

    @classmethod
    def addSettings(klass, s):
        """Construct list of settings."""
        plotters.GenericPlotter.addSettings(s)

        s.add( setting.DatasetExtended(
            'covyy', '',
            descr=_('Covariance matrix entry (Y,Y) [computed from data if blank]'),
            usertext=_('Cov(Y,Y)')), 0 )
        s.add( setting.DatasetExtended(
            'covxy', '',
            descr=_('Covariance matrix entry (X,Y) [computed from data if blank]'),
            usertext=_('Cov(X,Y)')), 0 )
        s.add( setting.DatasetExtended(
            'covyx', '',
            descr=_('Covariance matrix entry (Y,X) [computed from data if blank]'),
            usertext=_('Cov(Y,X)')), 0 )
        s.add( setting.DatasetExtended(
            'covxx', '',
            descr=_('Covariance matrix entry (X,X) [computed from data if blank]'),
            usertext=_('Cov(X,X)')), 0 )

        s.add( setting.DatasetExtended(
            'yData', 'y',
            descr=_('Y values, given by dataset, expression or list of values'),
            usertext=_('Y data')), 0 )
        s.add( setting.DatasetExtended(
            'xData', 'x',
            descr=_('X values, given by dataset, expression or list of values'),
            usertext=_('X data')), 0 )

        s.add( CovarianceLine(
            'Line',
            descr=_('Line'),
            usertext=_('Ellipse line')),
            pixmap='settings_plotline' )

        s.add( setting.PlotterFill(
            'Fill',
            descr=_('Fill'),
            usertext=_('Ellipse fill')),
            pixmap='settings_plotfillbelow' )

    def _computeCovFromData(self, data):
        """Compute a single covariance matrix given data."""

        minlen = min(len(data['xData']), len(data['yData']))
        xd = data['xData'][:minlen]
        yd = data['yData'][:minlen]
        finite = N.isfinite(xd) & N.isfinite(yd)
        xd = xd[finite]
        yd = yd[finite]

        if len(xd) < 2:
            cov = N.array([[0,0],[0,0]])
        else:
            cov = N.cov(xd, y=yd)

        data['xData'] = N.array([N.mean(xd)])
        data['yData'] = N.array([N.mean(yd)])
        data['covxx'] = N.array([cov[0,0]])
        data['covxy'] = N.array([cov[1,0]])
        data['covyx'] = N.array([cov[0,1]])
        data['covyy'] = N.array([cov[1,1]])

    def _computeEllipses(self):
        """Calculate points for ellipses."""

        s = self.settings
        d = self.document

        # cache existing value if dataset unchanged
        if self._changeset == d.changeset:
            return

        self._elpts = []

        minlen = 1e99
        data = {}
        anynone = False
        for attr in 'xData', 'yData', 'covxx', 'covxy', 'covyx', 'covyy':
            dataset = s.get(attr).getData(d)
            # needs to be defined
            if dataset is not None:
                ds = dataset.data
                minlen = min(minlen, len(ds))
                data[attr] = ds
            else:
                data[attr] = None
                anynone = True

        # if covariance matrix not provided, compute from xy data
        if ( data['xData'] is not None and data['yData'] is not None and
             s.get('covxx').isEmpty() and s.get('covxy').isEmpty() and
             s.get('covyx').isEmpty() and s.get('covyy').isEmpty() ):
            self._computeCovFromData(data)
            minlen=1
        elif anynone:
            # invalid
            return

        # chop to minimum length
        for attr in data:
            if minlen != len(data[attr]):
                data[attr] = data[attr][:minlen]

        # remove invalid values
        valid = N.all([N.isfinite(ds) for ds in data.values()], axis=0)
        for attr in data:
            data[attr] = data[attr][valid]

        # construct covariance matrices
        cov = N.column_stack((
            data['covxx'], data['covyx'], data['covxy'], data['covyy']
            )).reshape(minlen, 2, 2)

        # compute eigenvalues and vectors from covariance matrices
        try:
            eigvals, eigvecs = N.linalg.eig(cov)
        except N.linalg.LinAlgError:
            return

        # multiply vectors be sqrt eigenvalues (error is sqrt)
        sqrtvals = N.sqrt(eigvals)
        eigcomb = eigvecs * sqrtvals[:,:,None]

        # generate points in ellipse
        numsteps = s.Line.steps
        x = N.linspace(0, N.pi*2, numsteps, endpoint=False)
        f1, f2 = N.cos(x), N.sin(x)

        combv = f1*eigcomb[:,0,:,None] + f2*eigcomb[:,1,:,None]
        xpts = data['xData'][:,None] - combv[:,0,:]
        ypts = data['yData'][:,None] + combv[:,1,:]

        # funny covariance matrix does this
        if N.any(N.iscomplex(xpts)) or N.any(N.iscomplex(ypts)):
            return

        # now we have the points
        self._elpts = [xpts, ypts]

    def affectsAxisRange(self):
        """This widget provides range information about these axes."""
        s = self.settings
        return ( (s.xAxis, 'sx'), (s.yAxis, 'sy') )

    def getRange(self, axis, depname, axrange):
        """Return range of data."""

        self._computeEllipses()
        if not self._elpts:
            return

        vals = self._elpts[{'sx': 0, 'sy': 1}[depname]]
        if axis.settings.log:
            vals = N.clip(vals, 1e-99, 1e99)
        if len(vals) > 0:
            axrange[0] = min(axrange[0], vals.min())
            axrange[1] = max(axrange[1], vals.max())

    def dataDraw(self, painter, axes, posn, cliprect):
        """Draw dataset."""

        s = self.settings

        self._computeEllipses()
        if not self._elpts:
            return

        if axes[0].settings.log:
            self._elpts[0] = N.clip(self._elpts[0], 1e-99, 1e99)
        if axes[1].settings.log:
            self._elpts[1] = N.clip(self._elpts[1], 1e-99, 1e99)

        ptsx = axes[0].dataToPlotterCoords(posn, self._elpts[0])
        ptsy = axes[1].dataToPlotterCoords(posn, self._elpts[1])

        pen = s.Line.makeQPenWHide(painter)
        pw = pen.widthF()*2
        x1, y1, x2, y2 = posn
        lineclip = qt.QRectF(
            qt.QPointF(x1-pw, y1-pw), qt.QPointF(x2+pw, y2+pw))

        for xvals, yvals in zip(ptsx, ptsy):
            path = qt.QPainterPath()
            poly = qt.QPolygonF()
            utils.addNumpyToPolygonF(poly, xvals, yvals)
            clippedpoly = qt.QPolygonF()
            utils.polygonClip(poly, lineclip, clippedpoly)
            path.addPolygon(clippedpoly)
            path.closeSubpath()
            utils.brushExtFillPath(painter, s.Fill, path, stroke=pen)

document.thefactory.register(Covariance)



#    Copyright (C) 2006 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

"""Dialog to show if there is an error loading."""

from .. import qtall as qt
from .veuszdialog import VeuszDialog

class ErrorLoadingDialog(VeuszDialog):
    """Dialog when error loading."""

    def __init__(self, parent, filename, error, traceback):
        VeuszDialog.__init__(self, parent, 'errorloading.ui')

        # insert filename into label
        text = self.errorlabel.text()
        text = text % filename
        self.errorlabel.setText(text)
        self.errormessagelabel.setText(error)

        # put backtrace into error edit box
        self.errortextedit.setPlainText(traceback)

        # set warning pixmap to left of dialog
        icon = qt.qApp.style().standardIcon(
            qt.QStyle.SP_MessageBoxWarning, None, self)
        self.iconlabel.setPixmap(icon.pixmap(32))



import sys

import numpy as N
import veusz.embed as veusz

def main(outfile):
    # note - avoid putting text in here to avoid font issues

    embed = veusz.Embedded(hidden=True)
    x = N.arange(5)
    y = x**2
    embed.SetData('a', x)
    embed.SetData('b', y)

    page = embed.Root.Add('page')
    graph = page.Add('graph')
    xy = graph.Add('xy', xData='a', yData='b', marker='square')
    graph.x.TickLabels.hide.val = True
    graph.y.TickLabels.hide.val = True
    xy.MarkerFill.color.val = 'blue'

    embed.Export(outfile)

if __name__ == '__main__':
    main(sys.argv[1])



#    Copyright (C) 2016 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

"""Text datasets."""

from .commonfn import _
from .base import DatasetConcreteBase

from .. import utils

class DatasetText(DatasetConcreteBase):
    """Represents a text dataset: holding an array of strings."""

    dimensions = 1
    datatype = displaytype = 'text'
    columns = ('data',)
    column_descriptions = (_('Data'),)
    dstype = _('Text')
    editable = True

    def __init__(self, data=None, linked=None):
        """Initialise dataset with data given. Data are a list of strings."""

        DatasetConcreteBase.__init__(self, linked=linked)
        self.data = list(data)

    def description(self):
        return _('Text (length %i)') % len(self.data)

    def userSize(self):
        """Size of dataset."""
        return str( len(self.data) )

    def changeValues(self, type, vals):
        if type == 'data':
            self.data = list(vals)
        else:
            raise ValueError('type does not contain an allowed value')

        self.document.modifiedData(self)

    def uiConvertToDataItem(self, val):
        """Return a value cast to this dataset data type."""
        return str(val)

    def uiDataItemToData(self, val):
        """Return val converted to data."""
        return val

    def saveDataDumpToText(self, fileobj, name):
        '''Save data to file.
        '''
        fileobj.write("SetDataText(%s, [\n" % repr(name))
        for line in self.data:
            fileobj.write("    %s,\n" % repr(line))
        fileobj.write("])\n")

    def saveDataDumpToHDF5(self, group, name):
        """Save text data to hdf5 file."""
        tgrp = group.create_group(utils.escapeHDFDataName(name))
        tgrp.attrs['vsz_datatype'] = 'text'
        # make sure data are encoded
        encdata = [x.encode('utf-8') for x in self.data]
        tgrp['data'] = encdata
        tgrp['data'].attrs['vsz_name'] = name.encode('utf-8')

    def datasetAsText(self, fmt=None, join=None):
        """Return data as text."""
        lines = list(self.data)
        lines.append('')
        return '\n'.join(lines)

    def deleteRows(self, row, numrows):
        """Delete numrows rows starting from row.
        Returns deleted rows as a dict of {column:data, ...}
        """
        retn = {'data': self.data[row:row+numrows]}
        del self.data[row:row+numrows]

        self.document.modifiedData(self)
        return retn

    def insertRows(self, row, numrows, rowdata):
        """Insert numrows rows starting from row.
        rowdata is a dict of {column: data}.
        """
        data = rowdata.get('data', [])

        insdata = data + (['']*(numrows-len(data)))
        for d in insdata[::-1]:
            self.data.insert(row, d)

        self.document.modifiedData(self)

    def returnCopy(self):
        """Returns version of dataset with no linking."""
        return DatasetText(self.data)

    def returnCopyWithNewData(self, **args):
        """Return dataset of same type using the column data given."""
        return DatasetText(**args)



#    Copyright (C) 2016 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

from .. import qtall as qt
from .. import utils
from .. import document
from . import simpleread
from . import base

def _(text, disambiguation=None, context="Import_ND"):
    return qt.QCoreApplication.translate(context, text, disambiguation)

class ImportParamsND(base.ImportParamsBase):
    """nD import parameters.

     transpose: transpose array
     mode: text or csv
     csvdelimiter/csvtextdelimiter: csv text delimiters
     csvlocale: locale when importing csv
    """

    defaults = {
        'dataset': None,
        'datastr': None,
        'shape': None,
        'transpose': False,
        'mode': 'text',
        'csvdelimiter': ',',
        'csvtextdelimiter': '"',
        'csvlocale': 'en_US',
    }
    defaults.update(base.ImportParamsBase.defaults)

class LinkedFileND(base.LinkedFileBase):
    """Class representing a file linked to an nD dataset."""

    def createOperation(self):
        """Return operation to recreate self."""
        return OperationDataImportND

    def saveToFile(self, fileobj, relpath=None):
        """Save the link to the document file."""
        self._saveHelper(
            fileobj,
            'ImportFileND',
            ('filename', 'dataset'),
            relpath=relpath)

class OperationDataImportND(base.OperationDataImportBase):
    """Import an n-D matrix from a file."""

    descr = _('import nD data')

    def doImport(self):
        """Import data."""

        p = self.params

        # get stream
        if p.mode == 'csv':
            stream = simpleread.CSVStream(
                p.filename, p.csvdelimiter, p.csvtextdelimiter,
                p.csvlocale, p.encoding)
        elif p.filename is not None:
            stream = simpleread.FileStream(
                utils.openEncoding(p.filename, p.encoding) )
        elif p.datastr is not None:
            stream = simpleread.StringStream(p.datastr)
        else:
            raise RuntimeError("Invalid combination of parameters")

        # linked file
        LF = None
        if p.linked:
            assert p.filename
            LF = LinkedFileND(p)

        sr = simpleread.SimpleReadND(p.dataset, p)
        sr.readData(stream)
        sr.setOutput(self.outdatasets, linkedfile=LF)

def ImportFileND(
        comm, filename, dataset,
        shape=None,
        transpose=False,
        mode='text', csvdelimiter=',', csvtextdelimiter='"',
        csvlocale='en_US',
        prefix="", suffix="", encoding='utf_8',
        linked=False):

    """Import n-dimensional data from a file.
    filename is the name of the file to read
    dataset is the dataset to read

    if shape is set, the dataset is reshaped to these dimensions after loading
    if transpose=True, then rows and columns, etc, are swapped

    mode is either 'text' or 'csv'
    csvdelimiter is the csv delimiter for csv
    csvtextdelimiter is the csv text delimiter for csv
    csvlocale is locale to use when reading csv data

    prefix and suffix are prepended and appended to dataset names

    encoding is encoding character set

    if linked=True then the dataset is linked to the file

    Returns: list of imported datasets
    """

    # look up filename on path
    realfilename = comm.findFileOnImportPath(filename)

    params = ImportParamsND(
        dataset=dataset,
        filename=realfilename,
        transpose=transpose,
        mode=mode,
        csvdelimiter=csvdelimiter,
        csvtextdelimiter=csvtextdelimiter,
        csvlocale=csvlocale,
        prefix=prefix, suffix=suffix,
        linked=linked)
    op = OperationDataImportND(params)
    comm.document.applyOperation(op)

    if comm.verbose:
        print("Imported datasets %s" % ', '.join(op.outnames))
    return op.outnames

def ImportStringND(comm, dataset, dstring, shape=None, transpose=False):
    """Read n-dimensional data from the string specified.
    dataset is a dataset to read from the string

    if shape is set, then the array is reshaped to these dimensions
    if transpose=True, then rows and columns, etc, are swapped

    Returns: list of imported datasets
    """

    params = ImportParamsND(
        dataset=dataset,
        datastr=dstring,
        shape=shape,
        transpose=transpose)
    op = OperationDataImportND(params)
    comm.document.applyOperation(op)

    if comm.verbose:
        print("Imported datasets %s" % ', '.join(op.outnames))
    return op.outnames

document.registerImportCommand('ImportFileND', ImportFileND)
document.registerImportCommand(
    'ImportStringND', ImportStringND, filenamearg=-1)



#    Copyright (C) 2010 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

from .. import qtall as qt
from .. import utils
from .. import datasets
from .. import document

from .veuszdialog import VeuszDialog

def _(text, disambiguation=None, context="HistogramDialog"):
    """Translate text."""
    return qt.QCoreApplication.translate(context, text, disambiguation)

def checkValidator(combo):
    """Is this validator ok?"""
    valid = combo.validator()
    state, s, x = valid.validate(combo.currentText(), 0)
    return state == qt.QValidator.Acceptable

class ManualBinModel(qt.QAbstractListModel):
    """Model to store a list of floating point values in a list."""
    def __init__(self, thedata):
        qt.QAbstractListModel.__init__(self)
        self.thedata = thedata
    def data(self, index, role):
        if role == qt.Qt.DisplayRole and index.isValid():
            return float(self.thedata[index.row()])
        return None
    def rowCount(self, parent):
        return len(self.thedata)
    def flags(self, index):
        return (
            qt.Qt.ItemIsSelectable | qt.Qt.ItemIsEnabled |
            qt.Qt.ItemIsEditable )
    def setData(self, index, value, role):
        if role == qt.Qt.EditRole:
            try:
                val = float(value)
            except ValueError:
                return False

            self.thedata[ index.row() ] = val
            self.dataChanged.emit(index, index)
            return True
        return False

class HistoDataDialog(VeuszDialog):
    """Preferences dialog."""

    def __init__(self, parent, document):
        """Setup dialog."""
        VeuszDialog.__init__(self, parent, 'histodata.ui')
        self.document = document

        self.minval.default = self.maxval.default = ['Auto']
        regexp = qt.QRegExp(r"^[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?|Auto$")
        validator = qt.QRegExpValidator(regexp, self)
        self.minval.setValidator(validator)
        self.maxval.setValidator(validator)
        self.buttonBox.button(qt.QDialogButtonBox.Apply).clicked.connect(
            self.applyClicked )
        self.buttonBox.button(qt.QDialogButtonBox.Reset).clicked.connect(
            self.resetClicked )
        self.bingenerate.clicked.connect(self.generateManualBins)
        self.binadd.clicked.connect(self.addManualBins)
        self.binremove.clicked.connect(self.removeManualBins)

        self.bindata = []
        self.binmodel = ManualBinModel(self.bindata)
        self.binmanuals.setModel(self.binmodel)

        document.signalModified.connect(self.updateDatasetLists)
        self.updateDatasetLists()

    def escapeDatasets(self, dsnames):
        """Escape dataset names if they are not typical python ones."""

        for i in range(len(dsnames)):
            if not utils.validPythonIdentifier(dsnames[i]):
                dsnames[i] = '`%s`' % dsnames[i]

    def updateDatasetLists(self):
        """Update list of datasets."""

        datasets = []
        for name, ds in self.document.data.items():
            if ds.datatype == 'numeric' and ds.dimensions == 1:
                datasets.append(name)
        datasets.sort()

        # make sure names are escaped if they have funny characters
        self.escapeDatasets(datasets)

        # help the user by listing existing datasets
        utils.populateCombo(self.indataset, datasets)

    def datasetExprChanged(self):
        """Validate expression."""
        text = self.indataset.text()
        datasets.evalDatasetExpression(self.document, text)

    class Params:
        """Parameters to creation of histogram."""

        def __init__(self, dialog):
            """Initialise parameters from dialog."""
            numbins = dialog.numbins.value()

            if not checkValidator(dialog.minval):
                raise RuntimeError(_("Invalid minimum value"))
            minval = dialog.minval.text()
            if minval != 'Auto':
                minval = float(minval)

            if not checkValidator(dialog.maxval):
                raise RuntimeError(_("Invalid maximum value"))
            maxval = dialog.maxval.text()
            if maxval != 'Auto':
                maxval = float(maxval)

            islog = dialog.logarithmic.isChecked()
            self.binparams = (numbins, minval, maxval, islog)

            self.expr = dialog.indataset.currentText().strip()
            self.outdataset = dialog.outdataset.currentText().strip()
            self.outbins = dialog.outbins.currentText().strip()

            if self.expr == self.outdataset or self.expr == self.outbins:
                raise RuntimeError(_("Output datasets cannot be the same as input datasets"))

            self.method = dialog.methodGroup.getRadioChecked().objectName()
            self.manualbins = list( dialog.bindata )
            self.manualbins.sort()
            if len(self.manualbins) == 0:
                self.manualbins = None

            self.errors = dialog.errorBars.isChecked()
            cuml = dialog.cumlGroup.getRadioChecked().objectName()
            self.cumulative = 'none'
            if cuml == 'cumlStoL':
                self.cumulative = 'smalltolarge'
            elif cuml == 'cumlLtoS':
                self.cumulative = 'largetosmall'

        def getGenerator(self, doc):
            """Return dataset generator."""
            return datasets.DatasetHistoGenerator(
                doc, self.expr, binparams = self.binparams,
                binmanual = self.manualbins, method = self.method,
                cumulative = self.cumulative, errors = self.errors)

        def getOperation(self):
            """Get operation to make histogram."""
            return document.OperationDatasetHistogram(
                self.expr, self.outbins, self.outdataset,
                binparams = self.binparams,
                binmanual = self.manualbins,
                method = self.method,
                cumulative = self.cumulative,
                errors = self.errors)

    def generateManualBins(self):
        """Generate manual bins."""

        try:
            p = HistoDataDialog.Params(self)
        except RuntimeError as ex:
            qt.QMessageBox.warning(self, _("Invalid parameters"), str(ex))
            return

        self.binmodel.beginRemoveRows(qt.QModelIndex(), 0, len(self.bindata)-1)
        del self.bindata[:]
        self.binmodel.endRemoveRows()

        if p.expr != '':
            p.manualbins = []
            gen = p.getGenerator(self.document)
            locs = list(gen.binLocations())
            self.binmodel.beginInsertRows(qt.QModelIndex(), 0, len(locs)-1)
            self.bindata += locs
            self.binmodel.endInsertRows()

    def addManualBins(self):
        """Add an extra bin to the manual list."""
        self.binmodel.beginInsertRows(qt.QModelIndex(), 0, 0)
        self.bindata.insert(0, 0.)
        self.binmodel.endInsertRows()

    def removeManualBins(self):
        """Remove selected bins."""
        indexes = self.binmanuals.selectionModel().selectedIndexes()
        if indexes:
            row = indexes[0].row()
            self.binmodel.beginRemoveRows(qt.QModelIndex(), row, row)
            del self.bindata[row]
            self.binmodel.endRemoveRows()

    def resetClicked(self):
        """Reset button clicked."""

        for cntrl in (self.indataset, self.outdataset, self.outbins):
            cntrl.setEditText("")

        self.numbins.setValue(10)
        self.minval.setEditText("Auto")
        self.maxval.setEditText("Auto")
        self.logarithmic.setChecked(False)

        self.binmodel.beginRemoveRows(qt.QModelIndex(), 0, len(self.bindata)-1)
        del self.bindata[:]
        self.binmodel.endRemoveRows()

        self.errorBars.setChecked(False)
        self.counts.click()
        self.cumlOff.click()

    def reEditDataset(self, ds, dsname):
        """Re-edit dataset."""

        gen = ds.generator

        self.indataset.setEditText(gen.inexpr)

        # need to map backwards to get dataset names
        revds = dict( (a,b) for b,a in self.document.data.items() )
        self.outdataset.setEditText(revds.get(gen.valuedataset, ''))
        self.outbins.setEditText(revds.get(gen.bindataset, ''))

        # if there are parameters
        if gen.binparams:
            p = gen.binparams
            self.numbins.setValue( p[0] )
            self.minval.setEditText( str(p[1]) )
            self.maxval.setEditText( str(p[2]) )
            self.logarithmic.setChecked( bool(p[3]) )
        else:
            self.numbins.setValue(10)
            self.minval.setEditText("Auto")
            self.maxval.setEditText("Auto")
            self.logarithmic.setChecked(False)

        # if there is a manual list of bins
        if gen.binmanual is not None:
            self.binmodel.beginResetModel()
            self.bindata[:] = list(gen.binmanual)
            self.binmodel.endResetModel()

        # select correct method
        {
            'counts': self.counts, 'density': self.density,
            'fractions': self.fractions
        }[gen.method].click()

        # select if cumulative
        {
            'none': self.cumlOff, 'smalltolarge': self.cumlStoL,
            'largetosmall': self.cumlLtoS
        }[gen.cumulative].click()

        # if error bars
        self.errorBars.setChecked( bool(gen.errors) )

    def applyClicked(self):
        """Create histogram."""

        qt.QTimer.singleShot(4000, self.statuslabel.clear)
        try:
            p = HistoDataDialog.Params(self)
        except RuntimeError as ex:
            self.statuslabel.setText(_("Invalid parameters: %s") % str(ex))
            return

        exprresult = datasets.evalDatasetExpression(self.document, p.expr)
        if exprresult is None:
            self.statuslabel.setText(_("Invalid expression"))
            return

        op = p.getOperation()
        self.document.applyOperation(op)

        self.statuslabel.setText(
            _('Created datasets "%s" and "%s"') % (p.outbins, p.outdataset))

def recreateDataset(mainwindow, document, dataset, datasetname):
    """Open dialog to recreate histogram."""
    dialog = HistoDataDialog(mainwindow, document)
    mainwindow.showDialog(dialog)
    dialog.reEditDataset(dataset, datasetname)



#    Copyright (C) 2010 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

"""A paint engine for doing self-tests."""

from . import svg_export

class SelfTestPaintEngine(svg_export.SVGPaintEngine):
    """Paint engine class for self testing output."""

    def __init__(self):
        svg_export.SVGPaintEngine.__init__(self)
        # ppm images are simple and should be same on all platforms
        self.imageformat = 'ppm'

    def drawTextItem(self, pt, textitem):
        """Write text directly in self test mode."""

        text = textitem.text().encode('ascii', 'xmlcharrefreplace').decode(
            'ascii')
        svg_export.SVGElement(
            self.celement, 'text',
            'x="%s" y="%s" font-size="%gpt" fill="%s"' % (
                svg_export.fltStr(pt.x()),
                svg_export.fltStr(pt.y()),
                textitem.font().pointSize(),
                self.pen.color().name()
            ),
            text=text
        )

class SelfTestPaintDevice(svg_export.SVGPaintDevice):
     """Paint device for SVG paint engine.

     Note: this device is different to SVGPaintDevice because it
     switches scaling to 1 by default.
     """

     def __init__(self, fileobj, width_in, height_in, dpi=90):
         """Initialise with output file, and dimensions in inches."""
         svg_export.SVGPaintDevice.__init__(
             self, fileobj, width_in, height_in, dpi=dpi, scale=1)
         self.engine = SelfTestPaintEngine()



#    Copyright (C) 2011 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

"""A widget for navigating datasets."""

import os.path
import textwrap

import numpy as N

from .. import qtall as qt
from .. import setting
from .. import document
from .. import utils

from .lineeditwithclear import LineEditWithClear
from ..utils.treemodel import TMNode, TreeModel

def _(text, disambiguation=None, context="DatasetBrowser"):
    """Translate text."""
    return qt.QCoreApplication.translate(context, text, disambiguation)

def datasetLinkFile(ds):
    """Get a linked filename from a dataset."""
    return "/" if ds.linked is None else ds.linked.filename

def wrap(text, width):
    """Wrap text at columns. This needs to be split and rejoined."""
    lines = text.split("\n\n")
    out = [textwrap.fill(l, width).strip() for l in lines]
    return "\n\n".join(out)

class DatasetNode(TMNode):
    """Node for a dataset."""

    def __init__(self, model, dsname, cols, parent):
        ds = model.doc.data[dsname]
        data = []
        assert cols[0] == "name"
        for c in cols:
            if c == "name":
                data.append( dsname )
            elif c == "size":
                data.append( ds.userSize() )
            elif c == "type":
                data.append( ds.dstype )
            elif c == "linkfile":
                data.append( os.path.basename(datasetLinkFile(ds)) )
            elif c == "check":
                data.append( dsname in model.checked_datasets )

        TMNode.__init__(self, tuple(data), parent)
        self.model = model
        self.cols = cols
        self.dsname = dsname

    def getPreviewPixmap(self, ds):
        """Get a preview pixmap for a dataset."""
        size = (140, 70)
        if ds.dimensions != 1 or ds.datatype != "numeric":
            return None

        pixmap = qt.QPixmap(*size)
        pixmap.fill(qt.Qt.transparent)
        p = qt.QPainter(pixmap)
        p.setRenderHint(qt.QPainter.Antialiasing)

        # calculate data points
        try:
            if len(ds.data) < size[1]:
                y = ds.data
            else:
                intvl = len(ds.data)//size[1]+1
                y = ds.data[::intvl]
            x = N.arange(len(y))

            # plot data points on image
            minval, maxval = N.nanmin(y), N.nanmax(y)
            y = (y-minval) / (maxval-minval) * size[1]
            finite = N.isfinite(y)
            x, y = x[finite], y[finite]
            x = x * (1./len(x)) * size[0]

            poly = qt.QPolygonF()
            utils.addNumpyToPolygonF(poly, x, size[1]-y)
            p.setPen( qt.QPen(qt.Qt.blue) )
            p.drawPolyline(poly)

            # draw x axis if span 0
            p.setPen( qt.QPen(qt.Qt.black) )
            if minval <= 0 and maxval > 0:
                y0 = size[1] - (0-minval)/(maxval-minval)*size[1]
                p.drawLine(qt.QLineF(x[0], y0, x[-1], y0))
            else:
                p.drawLine(qt.QLineF(x[0], size[1], x[-1], size[1]))
            p.drawLine(qt.QLineF(x[0], 0, x[0], size[1]))

        except (ValueError, ZeroDivisionError):
            # zero sized array after filtering or min == max, so return None
            p.end()
            return None

        p.end()
        return pixmap

    def toolTip(self, column):
        """Return tooltip for column."""
        try:
            ds = self.model.doc.data[self.data[0]]
        except KeyError:
            return None

        c = self.cols[column]
        if c == "name":
            text = '%s: %s' % (self.data[0], ds.description())
            if ds.linked:
                text += '\n\n' + _('Linked to %s') % ds.linked.filename
            if ds.tags:
                text += '\n\n' + _('Tags: %s') % (' '.join(sorted(ds.tags)))
            return wrap(text, 40)
        elif c == "size" or (c == 'type' and 'size' not in self.cols):
            text = ds.userPreview()
            # add preview of dataset if possible
            pix = self.getPreviewPixmap(ds)
            if pix:
                text = text.replace("\n", "<br>")
                text = "<html>%s<br>%s</html>" % (text, utils.pixmapAsHtml(pix))
            return text
        elif c == "linkfile" or c == "type":
            return wrap(ds.linkedInformation(), 40)
        return None

    def dataset(self):
        """Get associated dataset."""
        try:
            return self.model.doc.data[self.data[0]]
        except KeyError:
            return None

    def datasetName(self):
        """Get dataset name."""
        return self.data[0]

    def cloneTo(self, newroot):
        """Make a clone of self at the root given."""
        return self.__class__(
            self.model, self.dsname, self.cols, newroot)

class FilenameNode(TMNode):
    """A special node for holding filenames of files."""

    def nodeData(self, column):
        """basename of filename for data."""
        if column == 0:
            if self.data[0] == "/":
                return "/"
            else:
                return os.path.basename(self.data[0])
        return None

    def filename(self):
        """Return filename."""
        return self.data[0]

    def toolTip(self, column):
        """Full filename for tooltip."""
        if column == 0:
            return self.data[0]
        return None

def treeFromList(nodelist, rootdata):
    """Construct a tree from a list of nodes."""
    tree = TMNode( rootdata, None )
    for node in nodelist:
        tree.insertChildSorted(node)
    return tree

class DatasetRelationModel(TreeModel):
    """A model to show how the datasets are related to each file."""
    def __init__(self, doc, grouping="filename", readonly=False,
                 filterdims=None, filterdtype=None,
                 checkable=False):
        """Model parameters:
        doc: document
        group: how to group datasets
        readonly: no modification of data
        filterdims/filterdtype: filter dimensions and datatypes.
        checkable: whether datasets are checkable
        """

        TreeModel.__init__(self, (_("Dataset"), _("Size"), _("Type")))
        self.doc = doc
        self.linkednodes = {}
        self.grouping = grouping
        self.filter = ""
        self.readonly = readonly
        self.filterdims = filterdims
        self.filterdtype = filterdtype
        self.checkable = checkable
        self.checked_datasets = set()
        self.refresh()

        doc.signalModified.connect(self.refresh)

    def datasetFilterOut(self, ds, node):
        """Should dataset be filtered out by filter options."""
        filterout = False

        # is filter text not in node text or text
        keep = True
        if self.filter != "":
            keep = False
            if any([t.find(self.filter) >= 0 for t in ds.tags
                    if isinstance(t, str)]):
                keep = True
            if any([t.find(self.filter) >= 0 for t in node.data
                    if isinstance(t, str)]):
                keep = True
        # check dimensions haven't been filtered
        if ( self.filterdims is not None and
             ds.dimensions not in self.filterdims ):
            filterout = True
        # check type hasn't been filtered
        if ( self.filterdtype is not None and
             ds.datatype not in self.filterdtype and
             'all' not in self.filterdtype):
            filterout = True

        if filterout:
            return True
        return not keep

    def makeGrpTreeNone(self):
        """Make tree with no grouping."""

        heads = [_("Dataset"), _("Size"), _("Type"), _("File")]
        cols = ["name", "size", "type", "linkfile"]
        if self.checkable:
            heads += [_('Select')]
            cols += [_('check')]

        tree = TMNode(heads , None)
        for name, ds in self.doc.data.items():
            child = DatasetNode(self, name, cols, None)

            # add if not filtered for filtering
            if not self.datasetFilterOut(ds, child):
                tree.insertChildSorted(child)
        return tree

    def makeGrpTree(self, coltitles, colitems, grouper, GrpNodeClass):
        """Make a tree grouping with function:
        coltitles: tuple of titles of columns for user
        colitems: tuple of items to lookup in DatasetNode
        grouper: function of dataset to return text for grouping
        GrpNodeClass: class for creating grouping nodes
        """

        if self.checkable:
            coltitles = coltitles + [_('Select')]
            colitems = colitems + [_('check')]

        grpnodes = {}
        for name, ds in self.doc.data.items():
            child = DatasetNode(self, name, colitems, None)

            # check whether filtered out
            if not self.datasetFilterOut(ds, child):
                # get group
                grps = grouper(ds)
                for grp in grps:
                    if grp not in grpnodes:
                        grpnodes[grp] = GrpNodeClass( (grp,), None )
                    # add to group
                    grpnodes[grp].insertChildSorted(child)

        return treeFromList(list(grpnodes.values()), coltitles)

    def makeGrpTreeFilename(self):
        """Make a tree of datasets grouped by linked file."""
        return self.makeGrpTree(
            [_("Dataset"), _("Size"), _("Type")],
            ["name", "size", "type"],
            lambda ds: (datasetLinkFile(ds),),
            FilenameNode
        )

    def makeGrpTreeSize(self):
        """Make a tree of datasets grouped by dataset size."""
        return self.makeGrpTree(
            [_("Dataset"), _("Type"), _("Filename")],
            ["name", "type", "linkfile"],
            lambda ds: (ds.userSize(),),
            TMNode
        )

    def makeGrpTreeType(self):
        """Make a tree of datasets grouped by dataset type."""
        return self.makeGrpTree(
            [_("Dataset"), _("Size"), _("Filename")],
            ["name", "size", "linkfile"],
            lambda ds: (ds.dstype,),
            TMNode
        )

    def makeGrpTreeTags(self):
        """Make a tree of datasets grouped by tags."""

        def getgrp(ds):
            if ds.tags:
                return sorted(ds.tags)
            else:
                return [_("None")]

        return self.makeGrpTree(
            [_("Dataset"), _("Size"), _("Type"), _("Filename")],
            ["name", "size", "type", "linkfile"],
            getgrp,
            TMNode
        )

    def flags(self, idx):
        """Return model flags for index."""
        f = TreeModel.flags(self, idx)
        # allow dataset names to be edited
        obj = self.objFromIndex(idx)
        if idx.isValid() and isinstance(obj, DatasetNode):
            col = idx.column()
            if not self.readonly and col == 0:
                # renameable dataset
                f |= qt.Qt.ItemIsEditable
            elif obj.cols[col] == "check":
                # checkable dataset
                f |= qt.Qt.ItemIsUserCheckable
        return f

    def setData(self, idx, val, role):
        """Rename dataset."""
        dsnode = self.objFromIndex(idx)
        name = dsnode.cols[idx.column()]

        if name == "name":
            if( not utils.validateDatasetName(val) or
                val in self.doc.data ):
                return False

            self.doc.applyOperation(
                document.OperationDatasetRename(dsnode.data[0], val))
            self.dataChanged.emit(idx, idx)
            return True

        elif name == "check":
            # update check box
            name = dsnode.dsname
            if val:
                self.checked_datasets.add(name)
            else:
                self.checked_datasets.remove(name)
            # emitted by refresh: self.dataChanged.emit(idx, idx)
            self.refresh()
            return True

    @qt.pyqtSlot()
    def refresh(self):
        """Update tree of datasets when document changes."""

        tree = {
            "none": self.makeGrpTreeNone,
            "filename": self.makeGrpTreeFilename,
            "size": self.makeGrpTreeSize,
            "type": self.makeGrpTreeType,
            "tags": self.makeGrpTreeTags,
        }[self.grouping]()

        self.syncTree(tree)

class DatasetsNavigatorTree(qt.QTreeView):
    """Tree view for dataset names."""

    updateitem = qt.pyqtSignal()
    selecteddatasets = qt.pyqtSignal(list)

    def __init__(self, doc, mainwin, grouping, parent,
                 readonly=False, filterdims=None, filterdtype=None,
                 checkable=False):
        """Initialise the dataset tree view.
        doc: veusz document
        mainwin: veusz main window (or None if readonly)
        grouping: grouping mode of datasets
        parent: parent window or None
        filterdims: if set, only show datasets with dimensions given
        filterdtype: if set, only show datasets with type given
        checkable: allow datasets to be selected
        """

        qt.QTreeView.__init__(self, parent)
        self.doc = doc
        self.mainwindow = mainwin
        self.model = DatasetRelationModel(
            doc, grouping, readonly=readonly,
            filterdims=filterdims,
            filterdtype=filterdtype,
            checkable=checkable)

        self.setModel(self.model)
        self.setSelectionMode(qt.QTreeView.ExtendedSelection)
        self.setSelectionBehavior(qt.QTreeView.SelectRows)
        self.setUniformRowHeights(True)
        self.setContextMenuPolicy(qt.Qt.CustomContextMenu)
        if not readonly:
            self.customContextMenuRequested.connect(self.showContextMenu)
        self.model.refresh()
        self.expandAll()

        # stretch of columns
        hdr = self.header()
        hdr.setStretchLastSection(False)
        hdr.setSectionResizeMode(0, qt.QHeaderView.Stretch)
        for col in range(1, 3):
            hdr.setSectionResizeMode(col, qt.QHeaderView.ResizeToContents)

        # when documents have finished opening, expand all nodes
        if mainwin is not None:
            mainwin.documentOpened.connect(self.expandAll)

        # keep track of selection
        self.selectionModel().selectionChanged.connect(self.slotNewSelection)

        # expand nodes by default
        self.model.rowsInserted.connect(self.slotNewRow)

    def changeGrouping(self, grouping):
        """Change the tree grouping behaviour."""
        self.model.grouping = grouping
        self.model.refresh()
        self.expandAll()

    def changeFilter(self, filtertext):
        """Change filtering text."""
        self.model.filter = filtertext
        self.model.refresh()
        self.expandAll()

    def selectDataset(self, dsname):
        """Find, and if possible select dataset name."""

        matches = self.model.match(
            self.model.index(0, 0, qt.QModelIndex()),
            qt.Qt.DisplayRole, dsname, -1,
            qt.Qt.MatchFixedString | qt.Qt.MatchCaseSensitive |
            qt.Qt.MatchRecursive )
        for idx in matches:
            if isinstance(self.model.objFromIndex(idx), DatasetNode):
                self.selectionModel().setCurrentIndex(
                    idx, qt.QItemSelectionModel.SelectCurrent |
                    qt.QItemSelectionModel.Clear |
                    qt.QItemSelectionModel.Rows )

    def showContextMenu(self, pt):
        """Context menu for nodes."""

        # get selected nodes
        idxs = self.selectionModel().selection().indexes()
        nodes = [ self.model.objFromIndex(i)
                  for i in idxs if i.column() == 0 ]

        # unique list of types of nodes
        types = set([ type(n) for n in nodes ])

        menu = qt.QMenu()
        # put contexts onto submenus if multiple types selected
        if DatasetNode in types:
            thismenu = menu
            if len(types) > 1:
                thismenu = menu.addMenu(_("Datasets"))
            self.datasetContextMenu(
                [n for n in nodes if isinstance(n, DatasetNode)],
                thismenu)
        elif FilenameNode in types:
            thismenu = menu
            if len(types) > 1:
                thismenu = menu.addMenu(_("Files"))
            self.filenameContextMenu(
                [n for n in nodes if isinstance(n, FilenameNode)],
                thismenu)

        def _paste():
            """Paste dataset(s)."""
            if document.isClipboardDataMime():
                mime = qt.QApplication.clipboard().mimeData()
                self.doc.applyOperation(document.OperationDataPaste(mime))

        # if there is data to paste, add menu item
        if document.isClipboardDataMime():
            menu.addAction(_("Paste"), _paste)

        if len( menu.actions() ) != 0:
            menu.exec_(self.mapToGlobal(pt))

    def datasetContextMenu(self, dsnodes, menu):
        """Return context menu for datasets."""
        from ..dialogs import dataeditdialog

        datasets = [d.dataset() for d in dsnodes]
        dsnames = [d.datasetName() for d in dsnodes]

        def _edit():
            """Open up dialog box to recreate dataset."""
            for dataset, dsname in zip(datasets, dsnames):
                if type(dataset) in dataeditdialog.recreate_register:
                    dataeditdialog.recreate_register[type(dataset)](
                        self.mainwindow, self.doc, dataset, dsname)
        def _edit_data():
            """Open up data edit dialog."""
            for dataset, dsname in zip(datasets, dsnames):
                if type(dataset) not in dataeditdialog.recreate_register:
                    self.mainwindow.slotDataEdit(editdataset=dsname)
        def _delete():
            """Simply delete dataset."""
            self.doc.applyOperation(
                document.OperationMultiple(
                    [document.OperationDatasetDelete(n) for n in dsnames],
                    descr=_('delete dataset(s)')))
        def _unlink_file():
            """Unlink dataset from file."""
            self.doc.applyOperation(
                document.OperationMultiple(
                    [document.OperationDatasetUnlinkFile(n)
                     for d,n in zip(datasets,dsnames)
                     if d.canUnlink() and d.linked],
                    descr=_('unlink dataset(s)')))
        def _unlink_relation():
            """Unlink dataset from relation."""
            self.doc.applyOperation(
                document.OperationMultiple(
                    [document.OperationDatasetUnlinkRelation(n)
                     for d,n in zip(datasets,dsnames)
                     if d.canUnlink() and not d.linked],
                    descr=_('unlink dataset(s)')))
        def _copy():
            """Copy data to clipboard."""
            mime = document.generateDatasetsMime(dsnames, self.doc)
            qt.QApplication.clipboard().setMimeData(mime)

        # editing
        recreate = [type(d) in dataeditdialog.recreate_register
                    for d in datasets]
        if any(recreate):
            menu.addAction(_("Edit"), _edit)
        if not all(recreate):
            menu.addAction(_("Edit data"), _edit_data)

        # deletion
        menu.addAction(_("Delete"), _delete)

        # linking
        unlink_file = [d.canUnlink() and d.linked for d in datasets]
        if any(unlink_file):
            menu.addAction(_("Unlink file"), _unlink_file)
        unlink_relation = [d.canUnlink() and not d.linked for d in datasets]
        if any(unlink_relation):
            menu.addAction(_("Unlink relation"), _unlink_relation)

        # tagging submenu
        tagmenu = menu.addMenu(_("Tags"))
        for tag in self.doc.datasetTags():
            def toggle(tag=tag):
                state = [tag in d.tags for d in datasets]
                if all(state):
                    op = document.OperationDataUntag
                else:
                    op = document.OperationDataTag
                self.doc.applyOperation(op(tag, dsnames))

            a = tagmenu.addAction(tag, toggle)
            a.setCheckable(True)
            state = [tag in d.tags for d in datasets]
            a.setChecked( all(state) )

        def addtag():
            tag, ok = qt.QInputDialog.getText(
                self, _("New tag"), _("Enter new tag"))
            if ok:
                tag = tag.strip().replace(' ', '')
                if tag:
                    self.doc.applyOperation( document.OperationDataTag(
                        tag, dsnames) )
        tagmenu.addAction(_("Add..."), addtag)

        # copy
        menu.addAction(_("Copy"), _copy)

        if len(datasets) == 1:
            useasmenu = menu.addMenu(_("Use as"))
            self.getMenuUseAs(useasmenu, datasets[0])

    def filenameContextMenu(self, nodes, menu):
        """Return context menu for filenames."""

        from ..dialogs.reloaddata import ReloadData

        filenames = [n.filename() for n in nodes if n.filename() != '/']
        if not filenames:
            return

        def _reload():
            """Reload data in this file."""
            d = ReloadData(self.doc, self.mainwindow, filenames=set(filenames))
            self.mainwindow.showDialog(d)
        def _unlink_all():
            """Unlink all datasets associated with file."""
            self.doc.applyOperation(
                document.OperationMultiple(
                    [document.OperationDatasetUnlinkByFile(f) for f in filenames],
                    descr=_('unlink by file')))
        def _delete_all():
            """Delete all datasets associated with file."""
            self.doc.applyOperation(
                document.OperationMultiple(
                    [document.OperationDatasetDeleteByFile(f) for f in filenames],
                    descr=_('delete by file')))

        menu.addAction(_("Reload"), _reload)
        menu.addAction(_("Unlink all"), _unlink_all)
        menu.addAction(_("Delete all"), _delete_all)

    def getMenuUseAs(self, menu, dataset):
        """Build up menu of widget settings to use dataset in."""

        def addifdatasetsetting(path, setn):
            def _setdataset():
                self.doc.applyOperation(
                    document.OperationSettingSet(
                        path, self.doc.datasetName(dataset)) )

            if ( isinstance(setn, setting.Dataset) and
                 setn.dimensions == dataset.dimensions and
                 setn.datatype == dataset.datatype and
                 path[:12] != "/StyleSheet/" ):
                menu.addAction(path, _setdataset)

        self.doc.walkNodes(addifdatasetsetting, nodetypes=("setting",))

    def keyPressEvent(self, event):
        """Enter key selects widget."""
        if event.key() in (qt.Qt.Key_Return, qt.Qt.Key_Enter):
            self.updateitem.emit()
            return
        qt.QTreeView.keyPressEvent(self, event)

    def mouseDoubleClickEvent(self, event):
        """Emit updateitem signal if double clicked."""
        retn = qt.QTreeView.mouseDoubleClickEvent(self, event)
        self.updateitem.emit()
        return retn

    def slotNewSelection(self, selected, deselected):
        """Emit selecteditem signal on new selection."""
        self.selecteddatasets.emit(self.getSelectedDatasets())

    def slotNewRow(self, parent, start, end):
        """Expand parent if added."""
        self.expand(parent)

    def getSelectedDatasets(self):
        """Returns list of selected datasets."""

        datasets = []
        for idx in self.selectionModel().selectedRows():
            node = self.model.objFromIndex(idx)
            try:
                name = node.datasetName()
                if name in self.doc.data:
                    datasets.append(name)
            except AttributeError:
                pass
        return datasets

    def checkedDatasets(self):
        """Returns list of checked datasets (if checkable was True)."""
        return sorted(self.model.checked_datasets)

    def setCheckedDatasets(self, checked):
        """Update list of checked datasets."""
        self.model.checked_datasets = set(checked)
        self.model.refresh()

    def resetChecks(self):
        """Reset checked datasets."""
        self.model.checked_datasets.clear()
        self.model.refresh()

class DatasetBrowser(qt.QWidget):
    """Widget which shows the document's datasets."""

    # how datasets can be grouped
    grpnames = ("none", "filename", "type", "size", "tags")
    grpentries = {
        "none": _("None"),
        "filename": _("Filename"),
        "type": _("Type"),
        "size": _("Size"),
        "tags": _("Tags"),
    }

    def __init__(self, thedocument, mainwin, parent, readonly=False,
                 filterdims=None, filterdtype=None, checkable=False):
        """Initialise widget:
        thedocument: document to show
        mainwin: main window of application (or None if readonly)
        parent: parent of widget.
        readonly: for choosing datasets only
        filterdims: if set, only show datasets with dimensions given
        filterdtype: if set, only show datasets with type given
        checkable: allow datasets to be selected
        """

        qt.QWidget.__init__(self, parent)
        self.layout = qt.QVBoxLayout()
        self.setLayout(self.layout)

        # options for navigator are in this layout
        self.optslayout = qt.QHBoxLayout()

        # grouping options - use a menu to choose the grouping
        self.grpbutton = qt.QPushButton(_("Group"))
        self.grpmenu = qt.QMenu()
        self.grouping = setting.settingdb.get("navtree_grouping", "filename")
        self.grpact = qt.QActionGroup(self)
        self.grpact.setExclusive(True)
        for name in self.grpnames:
            a = self.grpmenu.addAction(self.grpentries[name])
            a.grpname = name
            a.setCheckable(True)
            if name == self.grouping:
                a.setChecked(True)
            self.grpact.addAction(a)
        self.grpact.triggered.connect(self.slotGrpChanged)
        self.grpbutton.setMenu(self.grpmenu)
        self.grpbutton.setToolTip(_("Group datasets with property given"))
        self.optslayout.addWidget(self.grpbutton)

        # filtering by entering text
        searchlabel = qt.QLabel()
        searchlabel.setPixmap(utils.getIcon('kde-search-jss').pixmap(18,18))
        self.optslayout.addWidget(searchlabel)
        self.filteredit = LineEditWithClear()
        self.filteredit.setToolTip(_("Search for dataset names"))
        self.filteredit.textChanged.connect(self.slotFilterChanged)
        self.optslayout.addWidget(self.filteredit)

        self.layout.addLayout(self.optslayout)

        # the actual widget tree
        self.navtree = DatasetsNavigatorTree(
            thedocument, mainwin, self.grouping, None,
            readonly=readonly, filterdims=filterdims, filterdtype=filterdtype,
            checkable=checkable)
        self.layout.addWidget(self.navtree)

    def slotGrpChanged(self, action):
        """Grouping changed by user."""
        self.navtree.changeGrouping(action.grpname)
        setting.settingdb["navtree_grouping"] = action.grpname

    def slotFilterChanged(self, filtertext):
        """Filtering changed by user."""
        self.navtree.changeFilter(filtertext)

    def selectDataset(self, dsname):
        """Find, and if possible select dataset name."""
        self.navtree.selectDataset(dsname)

    def checkedDatasets(self):
        """Returns list of checked datasets (if checkable was True)."""
        return self.navtree.checkedDatasets()

    def setCheckedDatasets(self, checked):
        """Update list of checked datasets."""
        self.navtree.setCheckedDatasets(checked)

    def reset(self):
        """Uncheck all items."""
        self.navtree.resetChecks()
        self.filteredit.clear()

class DatasetBrowserPopup(DatasetBrowser):
    """Popup window for dataset browser for selecting datasets.
    This is used by setting.controls.Dataset
    """

    closing = qt.pyqtSignal()
    newdataset = qt.pyqtSignal(str)

    def __init__(self, document, dsname, parent,
                 filterdims=None, filterdtype=None):
        """Open popup window for document
        dsname: dataset name
        parent: window parent
        filterdims: if set, only show datasets with dimensions given
        filterdtype: if set, only show datasets with type given
        """

        DatasetBrowser.__init__(
            self, document, None, parent, readonly=True,
            filterdims=filterdims, filterdtype=filterdtype)
        self.setWindowFlags(qt.Qt.Popup)
        self.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.spacing = self.fontMetrics().height()

        utils.positionFloatingPopup(self, parent)
        self.selectDataset(dsname)
        self.installEventFilter(self)

        self.navtree.setFocus()

        self.navtree.updateitem.connect(self.slotUpdateItem)

    def eventFilter(self, node, event):
        """Grab clicks outside this window to close it."""
        if ( isinstance(event, qt.QMouseEvent) and
             event.buttons() != qt.Qt.NoButton ):
            frame = qt.QRect(0, 0, self.width(), self.height())
            if not frame.contains(event.pos()):
                self.close()
                return True
        return DatasetBrowser.eventFilter(self, node, event)

    def sizeHint(self):
        """A reasonable size for the text editor."""
        return qt.QSize(self.spacing*30, self.spacing*20)

    def closeEvent(self, event):
        """Tell the calling widget that we are closing."""
        self.closing.emit()
        event.accept()

    def slotUpdateItem(self):
        """Emit new dataset signal."""
        selected = self.navtree.selectionModel().currentIndex()
        if selected.isValid():
            n = self.navtree.model.objFromIndex(selected)
            if isinstance(n, DatasetNode):
                self.newdataset.emit(n.data[0])
                self.close()



#    Copyright (C) 2009 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

"""A combobox which remembers its history.

The history is stored in the Veusz settings database.
"""

from .. import qtall as qt
from .. import setting

class HistoryCombo(qt.QComboBox):
    """This combobox records what items have been entered into it so the
    user can choose them again.

    Duplicates and blanks are ignored.
    """

    def __init__(self, *args):
        qt.QComboBox.__init__(self, *args)

        # sane defaults
        self.setEditable(True)
        self.setMaxCount(50)
        self.setInsertPolicy(qt.QComboBox.InsertAtTop)
        self.setDuplicatesEnabled(False)
        self.setSizePolicy( qt.QSizePolicy(
            qt.QSizePolicy.MinimumExpanding, qt.QSizePolicy.Fixed) )

        # stops combobox readjusting in size to fit contents
        self.setSizeAdjustPolicy(
            qt.QComboBox.AdjustToMinimumContentsLengthWithIcon)

        self.default = []
        self.hasshown = False

    def text(self):
        """Get text in combobox
        - this gives it the same interface as QLineEdit."""
        return self.currentText()

    def setText(self, text):
        """Set text in combobox
        - gives same interface as QLineEdit."""
        self.lineEdit().setText(text)

    def hasAcceptableInput(self):
        """Input valid?
        - gives same interface as QLineEdit."""
        return self.lineEdit().hasAcceptableInput()

    def replaceAndAddHistory(self, item):
        """Replace the text and place item at top of history."""

        self.lineEdit().setText(item)
        index = self.findText(item)  # lookup for existing item (if any)
        if index != -1:
            # remove any old items matching this
            self.removeItem(index)

        # put new item in
        self.insertItem(0, item)
        # set selected item in drop down list match current item
        self.setCurrentIndex(0)

    def getSettingName(self):
        """Get name for saving in settings."""

        # get dialog for widget
        dialog = self.parent()
        while not isinstance(dialog, qt.QDialog):
            dialog = dialog.parent()

        # combine dialog and object names to make setting
        return '%s_%s_HistoryCombo'  % (
            dialog.objectName(), self.objectName() )

    def loadHistory(self):
        """Load contents of history combo from settings."""
        self.clear()
        history = setting.settingdb.get(self.getSettingName(), self.default)
        self.insertItems(0, history)

        self.hasshown = True

    def saveHistory(self):
        """Save contents of history combo to settings."""

        # only save history if it has been loaded
        if not self.hasshown:
            return

        # collect current items
        history = [ self.itemText(i) for i in range(self.count()) ]
        history.insert(0, self.currentText())

        # remove dups
        histout = []
        histset = set()
        for item in history:
            if item not in histset:
                histout.append(item)
                histset.add(item)

        # save the history
        setting.settingdb[self.getSettingName()] = histout

    def showEvent(self, event):
        """Show HistoryCombo and load history."""
        qt.QComboBox.showEvent(self, event)
        # we do this now rather than in __init__ because the widget
        # has no name set at __init__
        if not self.hasshown:
            self.loadHistory()

    def hideEvent(self, event):
        """Save history as widget is hidden."""
        qt.QComboBox.hideEvent(self, event)
        self.saveHistory()



#    Copyright (C) 2005 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

import os.path
import textwrap

from .. import qtall as qt
from . import utilfuncs

# where images are stored
imagedir = os.path.join(utilfuncs.resourceDirectory, 'icons')

_pixmapcache = {}
def getPixmap(pixmap):
    """Return a cached QPixmap for the filename in the icons directory."""
    if pixmap not in _pixmapcache:
        _pixmapcache[pixmap] = qt.QPixmap(os.path.join(imagedir, pixmap))
    return _pixmapcache[pixmap]

def pixmapExists(pixmap):
    """Does the pixmap exist?"""
    return (pixmap in _pixmapcache or
            os.path.exists(os.path.join(imagedir, pixmap)))

_iconcache = {}
def getIcon(icon):
    """Return a cached QIconSet for the filename in the icons directory."""
    if icon not in _iconcache:
        svg = os.path.join(imagedir, icon+'.svg')
        if os.path.exists(svg):
            filename = svg
        else:
            filename = os.path.join(imagedir, icon+'.png')

        _iconcache[icon] = qt.QIcon(filename)
    return _iconcache[icon]

def makeAction(parent, descr, menutext, slot, icon=None, key=None,
               checkable=False):
    """A quick way to set up an QAction object."""
    a = qt.QAction(parent)
    a.setText(menutext)
    a.setStatusTip(descr)
    a.setToolTip(textwrap.fill(descr, 25))
    if slot:
        a.triggered.connect(slot)
    if icon:
        a.setIcon(getIcon(icon))
    if key:
        a.setShortcut(qt.QKeySequence(key))
    if checkable:
        a.setCheckable(True)
    return a

def addToolbarActions(toolbar, actions, which):
    """Add actions listed in "which" from dict "actions" to toolbar "toolbar".
    """
    for w in which:
        toolbar.addAction(actions[w])

def constructMenus(rootobject, menuout, menutree, actions):
    """Add menus to the output dict from the tree, listing actions
    from actions.

    rootobject: QMenu or QMenuBar to add menus to
    menuout: dict to store menus
    menutree: tree structure to create menus from
    actions: dict of actions to assign to menu items
    """

    for menuid, menutext, actlist in menutree:
        # make a new menu if necessary
        if menuid not in menuout:
            menu = rootobject.addMenu(menutext)
            menuout[menuid] = menu
        else:
            menu = menuout[menuid]

        # add actions to the menu
        for action in actlist:
            if utilfuncs.isiternostr(action):
                # recurse for submenus
                constructMenus(menu, menuout, [action], actions)
            elif action == '':
                # blank means separator
                menu.addSeparator()
            else:
                # normal action
                menu.addAction(actions[action])

def populateMenuToolbars(items, toolbar, menus):
    """Construct the menus and toolbar from the list of items.
    toolbar is a QToolbar object
    menus is a dict of menus to add to

    Items are tuples consisting of:
    (actioname, status bar text, menu text, menu id, slot,
     icon filename, add to toolbar (bool), shortcut text)

    Returns a dict of actions
    """

    actions = {}
    parent = toolbar.parent()
    for item in items:
        if len(item) == 1:
            if menus is not None:
                menus[item[0]].addSeparator()
            continue

        menuid, descr, menutext, menu, slot, icon, addtool, key = item

        # create action
        action = qt.QAction(parent)
        action.setText(menutext)
        action.setStatusTip(descr)
        action.setToolTip(descr)

        # set shortcut if set
        if key:
            action.setShortcut(qt.QKeySequence(key))

        # load icon if set
        if icon:
            action.setIcon(getIcon(icon))

        if callable(slot):
            # connect the action to the slot
            action.triggered.connect(slot)
            # add to menu
            if menus is not None:
                menus[menu].addAction(action)
        elif slot is not None:
            if menus is not None:
                submenu = menus[menu].addMenu(menutext)
                menus["%s.%s" % (menu ,menuid)] = submenu
                populateMenuToolbars(slot, toolbar, menus)
        else:
            if menus is not None:
                menus[menu].addAction(action)

        # add to toolbar
        if addtool and toolbar is not None:
            toolbar.addAction(action)

        # save for later
        actions[menuid] = action

    return actions

def makeMenuGroupSaved(name, parent, actiondict, actionnames):
    """Assign a menu and group for an action which allows the user to
    choose from a list of other actions.

    name: name of action to assign to (in actiondict)
    parent: parent qt object
    actiondict: map of names to actions
    actionnames: name of actions to add to menu/action.
    """

    from .. import setting

    menu = qt.QMenu(parent)
    actgrp = qt.QActionGroup(parent)

    act_to_name = {}

    for actname in actionnames:
        action = actiondict[actname]
        act_to_name[action] = actname
        menu.addAction(action)
        actgrp.addAction(action)

    # assign menu to action in dictionary
    menuaction = actiondict[name]
    menuaction.setMenu(menu)

    # currently set value (per control)
    current = [
        setting.settingdb.get('menugrp_%s' % name, actionnames[0]) ]

    def ongrptriggered(action):
        """Update saved action when new one is chosen."""
        actname = act_to_name[action]
        setting.settingdb['menugrp_%s' % name] = current[0] = actname
        menuaction.setIcon(action.icon())

    def onactiontriggered():
        """Run the action chosen from the list."""
        actiondict[current[0]].trigger()

    menuaction.triggered.connect(onactiontriggered)
    actgrp.triggered.connect(ongrptriggered)
    menuaction.setIcon(actiondict[current[0]].icon())



#    Copyright (C) 2009 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##############################################################################

import select
import subprocess
import os
import socket
import platform
import signal

from .. import qtall as qt
from .. import utils
from . import simpleread

def _(text, disambiguation=None, context="Capture"):
    return qt.QCoreApplication.translate(context, text, disambiguation)

class CaptureFinishException(Exception):
    """An exception to say when a stream has been finished."""

class CaptureStream(simpleread.Stream):
    """A special stream for capturing data."""

    def __init__(self):
        """Initialise the stream."""

        simpleread.Stream.__init__(self)
        self.buffer = ''
        self.continuousreads = 0
        self.bytesread = 0
        self.linesread = 0
        self.maxlines = None
        self.timedout = False

    def _setTimeout(self, timeout):
        """Setter for setting timeout property."""
        if timeout:
            self.timer = qt.QTimer.singleShot(
                timeout*1000, self._timedOut)

    timeout = property(
        None, _setTimeout, None,
        "Time interval to stop in (seconds) or None")

    def _timedOut(self):
        self.timedout = True

    def getMoreData(self):
        """Override this to return more data from the source without
        blocking."""
        return ''

    def readLine(self):
        """Return a new line of data.

        Either returns new line or
        Raises StopIteration if there is no data, or more than 100 lines
        have been read."""

        while True:
            # we've reached the limit of lines or a timeout has occurred
            if self.linesread == self.maxlines:
                raise CaptureFinishException("Maximum number of lines read")
            if self.timedout:
                raise CaptureFinishException("Maximum time period occurred")

            # stop reading continous data greater than this many lines
            if self.continuousreads == 100:
                self.continuousreads = 0
                raise StopIteration

            index = self.buffer.find('\n')
            if index >= 0:
                # is there a line in the buffer?
                retn = self.buffer[:index]
                self.buffer = self.buffer[index+1:]
                self.linesread += 1
                self.continuousreads += 1
                return retn
            else:
                # if not, then read some more data
                data = self.getMoreData()

                if not data:
                    self.continuousreads = 0
                    raise StopIteration
                self.bytesread += len(data)
                self.buffer += data

    def close(self):
        """Close any allocated object."""
        pass

class FileCaptureStream(CaptureStream):
    """Capture from a file or named pipe."""

    def __init__(self, filename):
        CaptureStream.__init__(self)

        # open file
        self.fileobj = open(filename, 'rU')

        # make new thread to read file
        self.readerthread = utils.NonBlockingReaderThread(
            self.fileobj, exiteof=False)
        self.readerthread.start()

        self.name = filename

    def getMoreData(self):
        """Read data from the file."""
        try:
            data, done = self.readerthread.getNewData()
            if len(data) == 0 and done:
                raise CaptureFinishException("End of file")
            return data
        except OSError as e:
            raise CaptureFinishException("OSError: %s" % str(e))

    def close(self):
        """Close file."""
        self.fileobj.close()

class CommandCaptureStream(CaptureStream):
    """Capture from an external program."""

    def __init__(self, commandline):
        """Capture from commandline - this is passed to the shell."""
        CaptureStream.__init__(self)

        self.name = commandline
        self.popen = subprocess.Popen(
            commandline, shell=True,
            bufsize=0, stdout=subprocess.PIPE,
            universal_newlines=True)

        # make new thread to read stdout
        self.readerthread = utils.NonBlockingReaderThread(self.popen.stdout)
        self.readerthread.start()

    def getMoreData(self):
        """Read data from the command."""

        retn, done = self.readerthread.getNewData()

        if not retn:
            poll = self.popen.poll()
            if poll is not None:
                # process has ended
                raise CaptureFinishException(
                    "Process ended (status code %i)" % poll)
        return retn

    def close(self):
        """Close file."""

        if self.popen.poll() is None:
            # need to kill process if it is still running
            if platform.system() == 'Windows':
                # awful code (for windows)
                # use this to not have a ctypes dependency
                os.system('TASKKILL /PID %i /F' % self.popen.pid)
            else:
                # unix
                os.kill(self.popen.pid, signal.SIGTERM)

        try:
            self.popen.stdout.close()
        except EnvironmentError:
            # problems closing stdout for some reason
            pass

class SocketCaptureStream(CaptureStream):
    """Capture from an internet host."""

    def __init__(self, host, port):
        """Connect to host and port specified."""
        CaptureStream.__init__(self)

        self.name = '%s:%i' % (host, port)
        try:
            self.socket = socket.socket(
                socket.AF_INET, socket.SOCK_STREAM )
            self.socket.connect( (host, port) )
        except socket.error as e:
            self._handleSocketError(e)

    def _handleSocketError(self, e):
        """Special function to reraise exceptions
        because socket exceptions have changed in python 2.6 and
        behave differently on some platforms.
        """

        # clean up
        self.socket.close()

        # re-raise
        raise e

    def getMoreData(self):
        """Read data from the socket."""

        # see whether there is data to be read
        i, o, e = select.select([self.socket], [], [], 0)
        if i:
            try:
                retn = self.socket.recv(1024)
            except socket.error as e:
                self._handleSocketError(e)
            if len(retn) == 0:
                raise CaptureFinishException("Remote socket closed")
            return retn.decode('utf-8', errors='ignore')
        else:
            return ''

    def close(self):
        """Close the socket."""
        self.socket.close()

class OperationDataCaptureSet:
    """An operation for setting the results from a SimpleRead into the
    document's data from a data capture.

    This is a bit primative, but it is not obvious how to isolate the capturing
    functionality elsewhere."""

    descr = _('data capture')

    def __init__(self, simplereadobject):
        """Takes a simpleread object containing the data to be set."""
        self.simplereadobject = simplereadobject

    def do(self, doc):
        """Set the data in the document."""

        # set the data to the document and keep a list of what's changed
        readdata = {}
        self.simplereadobject.setOutput(readdata)

        # keep a copy of datasets which have changed from backup
        self.nameschanged = list(readdata)
        self.olddata = {}
        for name in self.nameschanged:
            if name in doc.data:
                self.olddata[name] = doc.data[name]
            doc.setData(name, readdata[name])

    def undo(self, doc):
        """Undo the results of the capture."""

        for name in self.nameschanged:
            if name in self.olddata:
                # replace datasets with what was there previously
                doc.setData(name, self.olddata[name])
            else:
                # or delete datasets that weren't there before
                doc.deleteData(name)



#    Copyright (C) 2009 Jeremy S. Sanders
#    Email: Jeremy Sanders <jeremy@jeremysanders.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
###############################################################################

import math
import datetime
import re

import numpy as N

# date format: YYYY-MM-DDTHH:MM:SS.mmmmmm
# date and time part are optional (check we have at least one!)
date_re = re.compile( r'''
^
# match date YYYY-MM-DD
([0-9]{4}-[0-9]{1,2}-[0-9]{1,2})?
[ ,A-Za-z]?
# match time HH:MM:SS
([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}(\.[0-9]+)?)?
$
''', re.VERBOSE )

# we store dates as intervals in sec from this date as a float
offsetdate = datetime.datetime(2009, 1, 1, 0, 0, 0, 0)
# this is the numpy version of this
offsetdate_np = N.datetime64('2009-01-01T00:00')

def isDateTime(datestr):
    """Check date/time string looks valid."""
    match = date_re.match(datestr)

    return match and (match.group(1) is not None or match.group(2) is not None)

def _isoDataStringToDate(datestr):
    """Convert ISO format date time to a datetime object."""
    match = date_re.match(datestr)
    val = None
    if match:
        try:
            dategrp, timegrp = match.group(1), match.group(2)
            if dategrp:
                # if there is a date part of the string
                dateval = [int(x) for x in dategrp.split('-')]
                if len(dateval) != 3:
                    raise ValueError("Invalid date '%s'" % dategrp)
            else:
                dateval = [2009, 1, 1]

            if timegrp:
                # if there is a time part of the string
                p = timegrp.split(':')
                if len(p) != 3:
                    raise ValueError("Invalid time '%s'" % timegrp)
                secfrac, sec = math.modf(float(p[2]))
                timeval = [ int(p[0]), int(p[1]), int(sec), int(secfrac*1e6) ]
            else:
                timeval = [0, 0, 0, 0]

            # either worked so return a datetime object
            if dategrp or timegrp:
                val = datetime.datetime( *(dateval+timeval) )

        except ValueError:
            # conversion failed, return nothing
            pass

    return val

def dateStringToDate(datestr):
    """Interpret a date string and return a Veusz-format date value."""
    dt = _isoDataStringToDate(datestr)
    if dt is None:
        # try local conversions (time, date and variations)
        # if ISO formats don't match
        for fmt in ('%X %x', '%x %X', '%x', '%X'):
            try:
                dt = datetime.datetime.strptime(datestr, fmt)
                break
            except (ValueError, TypeError):
                pass

    if dt is not None:
        delta = dt - offsetdate
        return (delta.days*24*60*60 + delta.seconds +
                delta.microseconds*1e-6)
    else:
        return N.nan

def floatUnixToVeusz(f):
    """Convert unix float to veusz float."""
    delta = datetime.datetime(1970,1,1) - offsetdate
    return f + delta.total_seconds()

def floatToDateTime(f):
    """Convert float to datetime."""
    days = int(f/24/60/60)
    frac, sec = math.modf(f - days*24*60*60)
    try:
        return datetime.timedelta(days,  sec,  frac*1e6) + offsetdate
    except OverflowError:
        return datetime.datetime(8000, 1, 1)

def dateFloatToString(f):
    """Convert date float to string."""
    if N.isfinite(f):
        return floatToDateTime(f).isoformat()
    else:
        return str(f)

def datetimeToTuple(dt):
    """Return tuple (year,month,day,hour,minute,second,microsecond) from
    datetime object."""
    return (dt.year, dt.month, dt.day, dt.hour, dt.minute,
            dt.second, dt.microsecond)

def datetimeToFloat(dt):
    """Convert datetime to float"""
    delta = dt - offsetdate
    # convert offset into a delta
    val = (delta.days*24*60*60 + (delta.seconds +
           delta.microseconds*1e-6))
    return val

def tupleToFloatTime(t):
    """Convert a tuple interval to a float style datetime"""
    dt = datetime.datetime(*t)
    return datetimeToFloat(dt)

def tupleToDateTime(t):
    """Convert a tuple to a datetime"""
    return datetime.datetime(*t)

def addTimeTupleToDateTime(dt,  tt):
    """Add a time tuple in the form (yr,mn,dy,h,m,s,us) to a datetime.
    Returns datetime
    """

    # add on most of the time intervals
    dt = dt + datetime.timedelta(
        days=tt[2], hours=tt[3],
        minutes=tt[4], seconds=tt[5],
        microseconds=tt[6]
    )

    # add on years
    dt = dt.replace(year=dt.year + tt[0])

    # add on months - this could be much simpler
    if tt[1] > 0:
        for i in range(tt[1]):
            # find interval between this month and next...
            m, y = dt.month + 1, dt.year
            if m == 13:
                m = 1
                y += 1
            dt = dt.replace(year=y, month=m)
    elif tt[1] < 0:
        for i in range(abs(tt[1])):
            # find interval between this month and next...
            m, y = dt.month - 1, dt.year
            if m == 0:
                m = 12
                y -= 1
            dt = dt.replace(year=y, month=m)

    return dt

def roundDownToTimeTuple(dt,  tt):
    """Take a datetime, and round down using the (yr,mn,dy,h,m,s,ms) tuple.
    Returns a tuple."""

    #print "round down",  dt,  tt
    timein = list(datetimeToTuple(dt))
    i = 6
    while i >= 0 and tt[i] == 0:
        if i == 1 or i == 2: # month, day
            timein[i] = 1
        else:
            timein[i] = 0
        i -= 1
    # round to nearest interval
    if (i == 1 or i == 2): # month, day
        timein[i] = ((timein[i]-1) // tt[i])*tt[i] + 1
    else:
        timein[i] = (timein[i] // tt[i])*tt[i]

    #print "rounded",  timein
    return tuple(timein)

def dateStrToRegularExpression(instr):
    """Convert date-time string to regular expression.

    Converts format yyyy-mm-dd|T|hh:mm:ss to re for date
    """

    # first rename each special string to a unique string (this is a
    # unicode character which is in the private use area) then rename
    # back again to the regular expression. This avoids the regular
    # expression being remapped.
    maps = (
        ('YYYY', '\ue001', r'(?P<YYYY>[0-9]{4})'),
        ('YY',   '\ue002', r'(?P<YY>[0-9]{2})'),
        ('MM',   '\ue003', r'(?P<MM>[0-9]{2})'),
        ('M',    '\ue004', r'(?P<MM>[0-9]{1,2})'),
        ('DD',   '\ue005', r'(?P<DD>[0-9]{2})'),
        ('D',    '\ue006', r'(?P<DD>[0-9]{1,2})'),
        ('hh',   '\ue007', r'(?P<hh>[0-9]{2})'),
        ('h',    '\ue008', r'(?P<hh>[0-9]{1,2})'),
        ('mm',   '\ue009', r'(?P<mm>[0-9]{2})'),
        ('m',    '\ue00a', r'(?P<mm>[0-9]{1,2})'),
        ('ss',   '\ue00b', r'(?P<ss>[0-9]{2}(\.[0-9]*)?)'),
        ('s',    '\ue00c', r'(?P<ss>[0-9]{1,2}(\.[0-9]*)?)'),
    )

    out = []
    for p in instr.split('|'):
        # escape special characters (non alpha-num)
        p = re.escape(p)

        # replace strings with characters
        for search, char, repl in maps:
            p = p.replace(search, char, 1)
        # replace characters with re strings
        for search, char, repl in maps:
            p = p.replace(char, repl, 1)

        # save as an optional group
        out.append( '(?:%s)?' % p )

    # return final expression
    return r'^\s*%s\s*$' % (''.join(out))

def dateREMatchToDate(match):
    """Take match object for above regular expression,
    and convert to float date value."""

    if match is None:
        raise ValueError("match object is None")

    # remove None matches
    grps = {}
    for k, v in match.groupdict().items():
        if v is not None:
            grps[k] = v

    # bomb out if nothing matches
    if len(grps) == 0:
        raise ValueError("no groups matched")

    # get values of offset
    oyear = offsetdate.year
    omon = offsetdate.month
    oday = offsetdate.day
    ohour = offsetdate.hour
    omin = offsetdate.minute
    osec = offsetdate.second
    omicrosec = offsetdate.microsecond

    # now convert each element from the re
    if 'YYYY' in grps:
        oyear = int(grps['YYYY'])
    if 'YY' in grps:
        y = int(grps['YY'])
        if y >= 70:
            oyear = int('19' + grps['YY'])
        else:
            oyear = int('20' + grps['YY'])
    if 'MM' in grps:
        omon = int(grps['MM'])
    if 'DD' in grps:
        oday = int(grps['DD'])
    if 'hh' in grps:
        ohour = int(grps['hh'])
    if 'mm' in grps:
        omin = int(grps['mm'])
    if 'ss' in grps:
        s = float(grps['ss'])
        osec = int(s)
        omicrosec = int(1e6*(s-osec))

    # convert to python datetime object
    d = datetime.datetime(
        oyear, omon, oday, ohour, omin, osec, omicrosec)

    # return to veusz float time
    return datetimeToFloat(d)
